
# Pyromaths
# -*- coding: utf-8 -*-
#
# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if notPopen, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

import random
#from ..outils.Affichage import decimaux

def noms_sommets(nb):
    """Renvoie nb noms de sommets"""

    (listenb, listepts) = ([], [])
    for i in range(26):
        listenb.append(i + 65)
    for i in range(nb):
        listepts.append(str(chr(listenb.pop(random.randrange(26 - i)))))
    listepts.sort()
    return listepts

def connect(pt,F1,F2):
    """renvoie les sommets connectes à pt"""
    if pt in F1:
        result=[F1[F1.index(pt)-1],F1[(F1.index(pt)+1)%len(F1)],F2[F1.index(pt)]]
    else:
        result=[F2[F2.index(pt)-1],F2[(F2.index(pt)+1)%len(F2)],F1[F2.index(pt)]]
    return result


def reponse1(pt,F1,F2):
    """renvoie la réponse à la première question"""
    conn=connect(pt,F1,F2)
    ptc1=conn[random.randrange(0,len(conn))]
    conn.remove(ptc1)
    ptc2=conn[random.randrange(0,len(conn))]
    conn.remove(ptc2)
    ptc3=conn[0]
    F3=[F2[0],F1[0],F1[3],F2[3]]
    F4=[F2[0],F2[1],F1[1],F1[0]]
    F5=[F1[1],F2[1],F2[2],F1[2]]
    F6=[F1[2],F2[2],F2[3],F1[3]]
    F2b=F2[:]
    F2b.reverse()
    if (pt in F1) and (ptc1 in F1) and (ptc2 in F1):
        i=F1.index(pt)
        j=F2.index(ptc3)
        res=[pt,F1[(i+1)%4],F1[(i+2)%4],F1[(i+3)%4],ptc3,F2[(j+1)%4],F2[(j+2)%4],F2[(j+3)%4]]
    elif (pt in F2) and (ptc1 in F2) and (ptc2 in F2):
        i=F2b.index(pt)
        j=F1.index(ptc3)
        res=[pt,F2b[(i+1)%4],F2b[(i+2)%4],F2b[(i+3)%4],ptc3,F1[(j-1)%4],F1[(j-2)%4],F1[(j-3)%4]]
    elif (pt in F3) and (ptc1 in F3) and (ptc2 in F3):
        i=F3.index(pt)
        j=F5.index(ptc3)
        res=[pt,F3[(i+1)%4],F3[(i+2)%4],F3[(i+3)%4],ptc3,F5[(j-1)%4],F5[(j-2)%4],F5[(j-3)%4]]
    elif (pt in F4) and (ptc1 in F4) and (ptc2 in F4):
        i=F4.index(pt)
        j=F6.index(ptc3)
        res=[pt,F4[(i+1)%4],F4[(i+2)%4],F4[(i+3)%4],ptc3,F6[(j-1)%4],F6[(j-2)%4],F6[(j-3)%4]]
    elif (pt in F5) and (ptc1 in F5) and (ptc2 in F5):
        i=F5.index(pt)
        j=F3.index(ptc3)
        res=[pt,F5[(i+1)%4],F5[(i+2)%4],F5[(i+3)%4],ptc3,F3[(j-1)%4],F3[(j-2)%4],F3[(j-3)%4]]
    elif (pt in F6) and (ptc1 in F6) and (ptc2 in F6):
        i=F6.index(pt)
        j=F4.index(ptc3)
        res=[pt,F6[(i+1)%4],F6[(i+2)%4],F6[(i+3)%4],ptc3,F5[(j-1)%4],F5[(j-2)%4],F5[(j-3)%4]]
    return res




def reponse2(pt1,ptc1,F1,F2):
    """renvoie la réponse à la deuxième question et le segment[p1ptc1]"""
    rep=[]
    connects=connect(pt1,F1,F2)
    for pt in connects:
        if pt<>ptc1:
            rep.append('['+pt1+pt+']')
    connects2=connect(ptc1,F1,F2)
    for pt in connects2:
        if pt<>pt1:
            rep.append('['+ptc1+pt+']')
    rep.append('['+pt1+ptc1+']')
    return tuple(rep)
def reponse3(pt2,ptc2,F1,F2):
    """renvoie la réponse à la troisièmequestion et le segment[pt2ptc2]"""
    if (pt2 in F1) and (ptc2 in F1):
        copy=F1[:]
        i=F1.index(pt2)
        j=F1.index(ptc2)
        copy.remove(pt2)
        copy.remove(ptc2)
        k=F1.index(copy[0])
        l=F1.index(copy[1])
        rep=('['+F2[i]+F2[j]+']','['+F1[k]+F1[l]+']','['+F2[k]+F2[l]+']','['+pt2+ptc2+']')
    elif (pt2 in F2) and (ptc2 in F2):
        copy=F2[:]
        i=F2.index(pt2)
        j=F2.index(ptc2)
        copy.remove(pt2)
        copy.remove(ptc2)
        k=F2.index(copy[0])
        l=F2.index(copy[1])
        rep=('['+F1[i]+F1[j]+']','['+F1[k]+F1[l]+']','['+F2[k]+F2[l]+']','['+pt2+ptc2+']')
    else:
        if pt2 in F1:
            i=F1.index(pt2)
        else:
            i=F2.index(pt2)
        rep=('['+F1[(i+1)%len(F1)]+F2[(i+1)%len(F1)]+']','['+F1[(i+2)%len(F1)]+F2[(i+2)%len(F1)]+']','['+F1[(i+3)%len(F1)]+F2[(i+3)%len(F1)]+']','['+pt2+ptc2+']')

    return tuple(rep)

def main():
    """Face 1, Face 2"""
    noms_pts=(noms_sommets(8))
    F1=noms_pts[0:4]
    F2=noms_pts[4:8]
    pt=noms_pts[random.randrange(0,len(noms_pts))]
    rp1=reponse1(pt,F1,F2)
    pt1=noms_pts[random.randrange(0,len(noms_pts))]
    conn1=connect(pt1,F1,F2)
    pt2= noms_pts[random.randrange(0,len(noms_pts))]
    conn2=connect(pt2,F1,F2)
    ptc1= conn1[random.randrange(0,len(conn1))]
    ptc2= conn2[random.randrange(0,len(conn2))]
    rp2=u"%s, %s, %s et %s sont les arêtes perpendiculaires à %s."%reponse2(pt1,ptc1,F1,F2)
    rp3=u"%s, %s et %s sont les arêtes parallèles à %s."%reponse3(pt2,ptc2,F1,F2)
    exo=["\\exercice",
         u"Les figures 1 et 2 représentent le même cube %s%s%s%s%s%s%s%s.\\\ "%tuple(noms_pts),
         "\\psset{xunit=1.0cm,yunit=1.0cm,dotstyle=*,dotsize=3pt 0,linewidth=0.8pt,arrowsize=3pt 2,arrowinset=0.25}",
         "\\begin{pspicture*}(-2,-0.38)(15,4.5)",
         "\\psframe[fillstyle=solid,fillcolor=darkgray,framearc=0.2](-1,3.5)(-0.5,4)",
         "\\rput[bl](-0.85,3.65){\\white{\\textbf{$1$}}}",
         "\\psline[linestyle=dashed,linecolor=Maroon,dash=4pt 4pt](3.5,1)(1.5,1)",
         "\\psline[linestyle=dashed,linecolor=Maroon,dash=4pt 4pt](1.5,1)(1.5,3)",
         "\\psline[linestyle=dashed,linecolor=Maroon,dash=4pt 4pt](1.5,1)(0.5,0)",
         "\\psframe[linecolor=Maroon](0.5,0)(2.5,2)",
         "\\psline[linecolor=Maroon](1.5,3)(3.5,3)",
         "\\psline[linecolor=Maroon](3.5,3)(3.5,1)",
         "\\psline[linecolor=Maroon](0.5,2)(1.5,3)",
         "\\psline[linecolor=Maroon](2.5,2)(3.5,3)",
         "\\psline[linecolor=Maroon](2.5,0)(3.5,1)",
         "\\rput[bl](0.22,2.08){%s}"%F1[0],
         "\\rput[bl](2.28,2.16){%s}"%F1[1],
         "\\rput[bl](2.6,-0.22){%s}"%F1[2],
         "\\rput[bl](0.15,-0.22){%s}"%F1[3],
         "\\rput[bl](1.4,3.08){%s}"%F2[0],
         "\\rput[bl](3.58,3.04){%s}"%F2[1],
         "\\rput[bl](3.58,1.04){%s}"%F2[2],
         "\\rput[bl](1.58,1.04){%s}"%F2[3],
         "\\psline[linecolor=Maroon](5.2,-04)(5.2,4)",
         "\\psframe[fillstyle=solid,fillcolor=darkgray,framearc=0.2](5.7,3.5)(6.2,4)",
         "\\rput[bl](5.85,3.65){\\white{\\textbf{$2$}}}",
         "\\psline[linecolor=Maroon](8.48,2.71)(7.18,2.42)",
         "\\psline[linecolor=Maroon](7.18,2.42)(7.18,0.46)",
         "\\psline[linestyle=dashed, linecolor=Maroon, dash=4pt 4pt](7.18,0.46)(8.48,0.75)",
         "\\psline[linestyle=dashed, linecolor=Maroon, dash=4pt 4pt](8.48,0.75)(8.48,2.71)",
         "\psline[linecolor=Maroon](10,2.46)(8.7,2.17)",
         "\\psline[linecolor=Maroon](8.7,2.17)(8.7,0.21)",
         "\\psline[linecolor=Maroon](8.7,0.21)(10,0.5)",
         "\\psline[linecolor=Maroon](10,0.5)(10,2.46)",
         "\\psline[linecolor=Maroon](10,2.46)(8.48,2.71)",
         "\\psline[linecolor=Maroon](7.18,2.42)(8.7,2.17)",
         "\\psline[linecolor=Maroon](8.7,0.21)(7.18,0.46)",
         "\\psline[linestyle=dashed,dash=4pt 4pt, linecolor=Maroon](10,0.5)(8.48,0.75)",
         "\\rput[bl](8.68,2.27){%s}"%rp1[0],
         "\\rput[bl](10.06,2.5){%s}"%rp1[1],
         "\\rput[bl](8.71,-0.1){%s}"%rp1[3],
         "\\end{pspicture*}",
         "\\begin{enumerate}",
         u"\\item Compléter les sommets manquants de la figure 2.",
         u"\\item Donner toutes les arêtes perpendiculaires à [%s%s]."%(pt1,ptc1),
         u"\\item Donner toutes les arêtes parallèles à [%s%s]."%(pt2,ptc2),
         "\\end{enumerate}"]
    cor=["\\exercice*",
         u"Les figures 1 et 2 représentent le même cube %s%s%s%s%s%s%s%s.\\\ "%tuple(noms_pts),
         "\\psset{xunit=1.0cm,yunit=1.0cm,dotstyle=*,dotsize=3pt 0,linewidth=0.8pt,arrowsize=3pt 2,arrowinset=0.25}",
         "\\begin{pspicture*}(-2,-0.38)(15,4.5)",
         "\\psframe[fillstyle=solid,fillcolor=darkgray,framearc=0.2](-1,3.5)(-0.5,4)",
         "\\rput[bl](-0.85,3.65){\\white{\\textbf{$1$}}}",
         "\\psline[linestyle=dashed,linecolor=Maroon,dash=4pt 4pt](3.5,1)(1.5,1)",
         "\\psline[linestyle=dashed,linecolor=Maroon,dash=4pt 4pt](1.5,1)(1.5,3)",
         "\\psline[linestyle=dashed,linecolor=Maroon,dash=4pt 4pt](1.5,1)(0.5,0)",
         "\\psframe[linecolor=Maroon](0.5,0)(2.5,2)",
         "\\psline[linecolor=Maroon](1.5,3)(3.5,3)",
         "\\psline[linecolor=Maroon](3.5,3)(3.5,1)",
         "\\psline[linecolor=Maroon](0.5,2)(1.5,3)",
         "\\psline[linecolor=Maroon](2.5,2)(3.5,3)",
         "\\psline[linecolor=Maroon](2.5,0)(3.5,1)",
         "\\rput[bl](0.22,2.08){%s}"%F1[0],
         "\\rput[bl](2.28,2.16){%s}"%F1[1],
         "\\rput[bl](2.6,-0.22){%s}"%F1[2],
         "\\rput[bl](0.15,-0.22){%s}"%F1[3],
         "\\rput[bl](1.4,3.08){%s}"%F2[0],
         "\\rput[bl](3.58,3.04){%s}"%F2[1],
         "\\rput[bl](3.58,1.04){%s}"%F2[2],
         "\\rput[bl](1.58,1.04){%s}"%F2[3],
         "\\psline[linecolor=Maroon](5.2,-04)(5.2,4)",
         "\\psframe[fillstyle=solid,fillcolor=darkgray,framearc=0.2](5.7,3.5)(6.2,4)",
         "\\rput[bl](5.85,3.65){\\white{\\textbf{$2$}}}",
         "\\psline[linecolor=Maroon](8.48,2.71)(7.18,2.42)",
         "\\psline[linecolor=Maroon](7.18,2.42)(7.18,0.46)",
         "\\psline[linestyle=dashed,dash=4pt 4pt, linecolor=Maroon](7.18,0.46)(8.48,0.75)",
         "\\psline[linestyle=dashed,dash=4pt 4pt, linecolor=Maroon](8.48,0.75)(8.48,2.71)",
         "\psline[linecolor=Maroon](10,2.46)(8.7,2.17)",
         "\\psline[linecolor=Maroon](8.7,2.17)(8.7,0.21)",
         "\\psline[linecolor=Maroon](8.7,0.21)(10,0.5)",
         "\\psline[linecolor=Maroon](10,0.5)(10,2.46)",
         "\\psline[linecolor=Maroon](10,2.46)(8.48,2.71)",
         "\\psline[linecolor=Maroon](7.18,2.42)(8.7,2.17)",
         "\\psline[linecolor=Maroon](8.7,0.21)(7.18,0.46)",
         "\\psline[linestyle=dashed,dash=4pt 4pt, linecolor=Maroon](10,0.5)(8.48,0.75)",
         "\\rput[bl](10.05,0.2){%s}"%rp1[2],
         "\\rput[bl](8.71,-0.1){%s}"%rp1[3],
         "\\rput[bl](8.1,0.79){%s}"%rp1[6],
         "\\rput[bl](6.8,0.2){%s}"%rp1[7],
         "\\rput[bl](10.06,2.5){%s}"%rp1[1],
         "\\rput[bl](8.68,2.27){%s}"%rp1[0],
         "\\rput[bl](7.08,2.54){%s}"%rp1[4],
         "\\rput[bl](8.5,2.83){%s}"%rp1[5],
         "\\end{pspicture*}",
         "\\begin{enumerate}",
         u"\\item Compléter les sommets manquants de la figure 2.",
         u"\\item Donner toutes les arêtes perpendiculaires à [%s%s].\\par "%(pt1,ptc1),
         rp2,
         u"\\item Donner toutes les arêtes parallèles à [%s%s]. \\par "%(pt2,ptc2),
         rp3,
         "\\end{enumerate}"]
    return (exo,cor)






#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#

import re
from .decimaux import decimaux
from ..classes import Racine, Fractions

        
def tex_coef(coef, var, bplus=0, bpn=0, bpc=0):
    """
    coef est le coefficient à écrire devant la variable var
    bplus est un booleen : s'il est vrai, il faut ecrire le signe +
    bpn est un booleen : s'il est vrai, il faut mettre des parentheses autour de
        l'écriture si coef est negatif.
    bpc est un booleen : s'il est vrai, il faut mettre des parentheses autour de
        l'écriture si coef =! 0 ou 1 et var est non vide
    """
    if coef != 0 and abs(coef) != 1:
        if var == '':
                a = TeX(coef)
        else:
                a = '%s\\,%s' % (TeX(coef), var)
        if bplus and coef > 0:
            a = '+' + a
    elif coef == 1:
        if var == '':
            a = '1'
        else:
            a = '%s' % var
        if bplus:
            a = '+' + a
    elif coef == 0:
        a = ''
    elif coef == -1:
        if var == '':
            a = '-1'
        else:
            a = '-%s' % var
    if bpn and coef < 0 or bpc and coef != 0 and coef != 1 and var != '':
        a = '\\left( ' + a + '\\right)'
    return a


def TeXz(nombre):
    '''n'affiche pas b si b=0'''
    if nombre==0:
        return ""
    else:
        return TeX(nombre)
def tTeX(nombre):
    '''raccourci pour TeX(nombre,terme=True)'''
    return TeX(nombre,terme=True)
def pTeX(nombre):
    '''raccourci pour TeX(nombre,parenthese=True)'''
    return TeX(nombre,parenthese=True)
def fTeX(nombre):
    '''raccourci pour TeX(nombre,fractex="\\frac")'''
    return TeX(nombre,fractex="\\frac")
    
def TeX(nombre,parenthese=False,terme=False,fractex="\\dfrac"):
    '''renvoie une chaine de caractere pour TeX'''
    strTeX=finTeX=""
    
    #Affichage simplifié des racines ou fractions
    if isinstance(nombre,Racine.RacineDegre2) and nombre.radicande==0:
        #Affiche la RacineDegre2 comme une Fractions
        nombre=Fractions.Fractions(nombre.numerateur,nombre.denominateur)
    if isinstance(nombre,Fractions.Fractions) and nombre.denominateur==1:
        #Affiche la Fractions comme un entier
        nombre=nombre.numerateur
    #parentheses des fractions
    if parenthese and (
        isinstance(nombre,Racine.RacineDegre2)
                       and nombre.denominateur==1 and (nombre.numerateur or nombre.coeff<0 )
        #RacineDegre2 avec radicande nécessairement grâce au tri
        or isinstance(nombre,Fractions.Fractions) and nombre.numerateur<0
        or isinstance(nombre,int) and nombre<0
        or isinstance(nombre,float) and nombre<0):
        strTeX="\\left("
        finTeX="\\right)" 
    elif terme and (isinstance(nombre,Racine.RacineDegre2) and
                        (nombre.denominateur!=1 or (nombre.numerateur >0 or nombre.numerateur==0 and nombre.coeff>=0))
                    or nombre>=0) :
        strTeX="+"
        finTeX=""

    ##Affichage
    if nombre==float("inf"):
        return "+\\infty "
    elif nombre==float("-inf"):
        return "-\\infty "
    elif isinstance(nombre,int) or isinstance(nombre,float):
        return strTeX+decimaux(nombre)+finTeX
    elif isinstance(nombre,Fractions.Fractions):
        if nombre.numerateur < 0:
            strTeX += "-"+fractex+"{"+decimaux(-nombre.numerateur)+"}{"+decimaux(nombre.denominateur)+"} "
        else:
            strTeX += fractex+"{"+decimaux(nombre.numerateur)+"}{"+decimaux(nombre.denominateur)+"} "
        strTeX+=finTeX
        return strTeX
    elif isinstance(nombre,Racine.RacineDegre2):
        return strTeX+str(nombre)+finTeX
    else:
        return strTeX+str(nombre)+finTeX
def radicalTeX(n):
    return "\\sqrt{%s}"%(decimaux(n))



# -*- coding: utf-8 -*-
#
# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#

import random
from math import *
from ..outils.Fractions import Fractions

def proba(exo, cor):
    couleur=['bleue','rouge','jaune','verte','marron','orange']
    initiale=['B','R','J','V','M','O']
    #Choix des 3 couleurs et du nombre de boule
    rg=random.randrange(0,4)
    c1,i1,n1=couleur[rg],initiale[rg],random.randrange(1,6) #1ere couleur, son initiale et le nombre
    #2e couleur diférente de la première
    rg2=(rg+random.randrange(1,5))%6
    c2,i2,n2=couleur[rg2],initiale[rg2],random.randrange(1,6)
    #3e couleur différente des précédentes
    c3,i3,n3=couleur[(rg2+1)%6],initiale[(rg2+1)%6],random.randrange(1,6)
    if n1>1:
        plur1="s"
    else :
        plur1=""
    if n2>1:
        plur2="s"
    else :
        plur2=""
    if n3>1:
        plur3="s"
    else :
        plur3=""
    exos=[u"Dans une urne, il y a %s boule%s %s%s (%s), %s boule%s %s%s (%s) et %s boule%s %s%s (%s), indiscernables au toucher. On tire successivement et sans remise deux boules."%(n1,plur1,c1,plur1,i1,n2,plur2,c2,plur2,i2,n3,plur3,c3,plur3,i3),
    "\\begin{enumerate}",
    u"\\item Quelle est la probabilité de tirer une boule %s au premier tirage?" %c2,
    u"\\item Construire un arbre des probabilités décrivant l'expérience aléatoire.",
    u"\\item Quelle est la probabilité que la première boule soit %s et la deuxième soit %s?"%(c3,c2),
    u"\\item Quelle est la probabilité que la deuxième boule soit %s ?"%c1,
    "\\end{enumerate}"]
    tot=n1+n2+n3
    #calculs intermédiaires pour la question 4
    p41="\\dfrac{%s}{%s}\\times \\dfrac{%s}{%s}+"%(n1,tot,n1-1,tot-1)
    p42="\\dfrac{%s}{%s}\\times \\dfrac{%s}{%s}+"%(n2,tot,n1,tot-1)
    p43="\\dfrac{%s}{%s}\\times \\dfrac{%s}{%s}="%(n3,tot,n1,tot-1)

    result4="\\dfrac{%s}{%s}"%(n1*(n1-1+n2+n3),tot*(tot-1)) #resultat non simplifié de la question 4
    cors=[u"Dans une urne, il y a %s boule%s %s%s (%s), %s boule%s %s%s (%s) et %s boule%s %s%s (%s), indiscernables au toucher. On tire successivement et sans remise deux boules."%(n1,plur1,c1,plur1,i1,n2,plur2,c2,plur2,i2,n3,plur3,c3,plur3,i3),

         "\\begin{enumerate}",
         u"\\item Quelle est la probabilité de tirer une boule %s au premier tirage?\\par " %c2,
         "Il y a %s boules dans l'urne dont %s boule%s %s%s. \\par"%(tot,n2,plur2,c2,plur2),
         u" La probabilité de tirer une boule %s au premier tirage est donc $\\dfrac{%s}{%s}$."%(c2,n2,tot),
         u"\\item Construire un arbre des probabilités décrivant l'expérience aléatoire.\\\ [0,3cm] ",
         "\\psset{unit=1 mm}",
         "\\psset{linewidth=0.3,dotsep=1,hatchwidth=0.3,hatchsep=1.5,shadowsize=1,dimen=middle}",
         "\\psset{dotsize=0.7 2.5,dotscale=1 1,fillcolor=black}",
         "\\psset{arrowsize=1 2,arrowlength=1,arrowinset=0.25,tbarsize=0.7 5,bracketlength=0.15,rbracketlength=0.15}",
         "\\begin{pspicture}(0,0)(80,53)",
         "\\psline(0,3)(10,23)",
         "\\psline(10,3)(10,23)",
         "\\psline(20,3)(10,23)",
         "\\psline(30,3)(40,23)",
         "\\psline(40,3)(40,23)",
         "\\psline(50,3)(40,23)",
         "\\psline(60,3)(70,23)",
         "\\psline(80,3)(70,23)",
         "\\psline(70,3)(70,23)",
         "\\psline(15,28)(40,53)",
         "\\psline(40,53)(65,28)",
         "\\psline(40,53)(40,28)",
         "\\rput(20,40){$\\dfrac{%s}{%s}$} \\rput(37,40){$\\dfrac{%s}{%s}$} \\rput(60,40){$\\dfrac{%s}{%s}$}"%(n1,tot,n2,tot,n3,tot),
         "\\rput(10,26){%s} \\rput(40,26){%s} \\rput(70,26){%s}"%(i1,i2,i3),
         "\\rput(0,10){$\\dfrac{%s}{%s}$} \\rput(7,10){$\\dfrac{%s}{%s}$} \\rput(20,10){$\\dfrac{%s}{%s}$}"%(n1-1,tot-1,n2,tot-1,n3,tot-1),
         "\\rput(0,0){%s} \\rput(10,0){%s} \\rput(20,0){%s}"%(i1,i2,i3),
         "\\rput(30,10){$\\dfrac{%s}{%s}$} \\rput(37,10){$\\dfrac{%s}{%s}$} \\rput(50,10){$\\dfrac{%s}{%s}$}"%(n1,tot-1,n2-1,tot-1,n3,tot-1),
         "\\rput(30,0){%s} \\rput(40,0){%s} \\rput(50,0){%s}"%(i1,i2,i3),
         "\\rput(60,10){$\\dfrac{%s}{%s}$} \\rput(67,10){$\\dfrac{%s}{%s}$} \\rput(80,10){$\\dfrac{%s}{%s}$}"%(n1,tot-1,n2,tot-1,n3-1,tot-1),
         "\\rput(60,0){%s} \\rput(70,0){%s} \\rput(80,0){%s}"%(i1,i2,i3),
         "\\end{pspicture}",
         "\\vspace{0.3cm}",
         u"\\item Quelle est la probabilité que la première boule soit %s et la deuxième soit %s?\\par"%(c3,c2),
         u"On utilise l'arbre construit précédemment.\\par",
         "$p(%s,%s)=%s \\times %s = %s$\\par" % \
                (i3,i2,Fractions.TeX(Fractions(n3,tot)),
                 Fractions.TeX(Fractions(n2,tot-1)),
                 Fractions.TeX(Fractions(n3*n2,tot*(tot-1)))),
         u"La probabilité que la première boule soit %s et la deuxième soit %s est égale à $\\dfrac{%s}{%s}$."%(c3,c2,n3*n2,tot*(tot-1)),
         u"\\item Quelle est la probabilité que la deuxième boule soit %s ?\\par"%c1,
         u"On note (?, %s) l'évènement: la deuxième boule tirée est %s. \\par"%(i1,c1),
         "$p(?,%s)=p(%s,%s)+p(%s,%s)+p(%s,%s,)="%(i1,i1,i1,i2,i1,i3,i1)+p41+p42+p43+result4+"$",
         "\\end{enumerate}"]
    for st in exos:
        exo.append(st)
    for st in cors:
        cor.append(st)
        
def tex_proba():
    exo=['\\exercice']
    cor=['\\exercice*']
    proba(exo,cor)
    return (exo, cor)




#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#

import string
import math
import os
from ..outils import Arithmetique

class Fractions:

    """Classe permettant d'opérer sur les fractions.
    Une fraction est stockée ainsi : (numérateur, dénominateur)"""

    def __init__(self, n, d=1):
        self.numerateur = self.n = n
        self.denominateur = self.d = d


    def simplifie(self):

        # retourne la fraction rendue irréductible

        pgcd = Arithmetique.pgcd(self.n, self.d)
        return Fractions(self.n // pgcd, self.d // pgcd)

    def __str__(self):
        return self.TeX(signe=0,coef=None)
    def TeX(self, signe=0, coef=None):
        """Permet d'écrire une fraction au format TeX.

        @param signe: Si vrai, écrit la fraction avec un dénominateur positif
        @param coef: Multiplie le numérateur et le dénominateur par un même nombre
        """

        if signe:
            (self.n, self.d) = ((self.n * abs(self.d)) // self.d, abs(self.d))
        if self.n:
            if coef and coef != 1:
                text = "\\dfrac{%s_{\\times %s}}{%s_{\\times %s}}" % (self.n,
                        coef, self.d, coef)
            elif self.d != 1:
                text = "\\dfrac{%s}{%s}" % (self.n, self.d)
            else:
                text = "%s" % self.n
        else:
            text = "0"
        return text

    def TeXProduit(self, fraction):
        if self.d < 0:
            (self.n, self.d) = (-self.n, -self.d)
        if fraction.d < 0:
            (fraction.n, fraction.d) = (-fraction.n, -fraction.d)
        c1 = abs(Arithmetique.pgcd(self.n, fraction.d))
        c2 = abs(Arithmetique.pgcd(self.d, fraction.n))
        simplifiable = 0  # permet de savoir si on a simplifiée le produit
        if c1 > 1:
            n1 = "%s \\times \\cancel{%s}" % (self.n // c1, c1)
            d2 = "%s \\times \\cancel{%s}" % (fraction.d // c1, c1)
        else:
            n1 = self.n
            d2 = fraction.d
        if c2 > 1:
            d1 = "%s \\times \\bcancel{%s}" % (self.d // c2, c2)
            n2 = "%s \\times \\bcancel{%s}" % (fraction.n // c2, c2)
        else:
            d1 = self.d
            n2 = fraction.n
        return "%s \\times %s" % (Fractions.TeX(Fractions(n1, d1), signe=
                                  None), Fractions.TeX(Fractions(n2, d2),
                                  signe=None))

    def TeXSimplifie(self):
        frs = Fractions.simplifie(self)
        coef = abs(self.n // frs.n)
        if coef > 1:
            texte = \
                "\\dfrac{%s_{\\times \\cancel %s}}{%s_{\\times \\cancel %s}}" % \
                (self.n // coef, coef, self.d // coef, coef)
        else:
            texte = Fractions.TeX(self, signe=None)
        return texte

    def __add__(self, fraction):
        if (isinstance(fraction,int) or isinstance(fraction,float)):
            fraction=Fractions(fraction)
        if isinstance(fraction,Fractions):
            ppcm = Arithmetique.ppcm(self.d, fraction.d)
            return Fractions((self.n * ppcm) // self.d + (fraction.n * ppcm) //
                         fraction.d, ppcm)
        else:
            return fraction+self

    def __radd__(self,other):
        return self+other

    def __sub__(self, fraction):
        return self + (-fraction)
    def __rsub__(self,other):
        return (-self) + other

    def __mul__(self, fraction):
        if (isinstance(fraction,int) or isinstance(fraction,float)):
            fraction=Fractions(fraction)
        if isinstance(fraction,Fractions):
            return Fractions(self.n * fraction.n, self.d * fraction.d)
        else:
            return fraction*self

    def __rmul__(self,other):
        return self*other

    #def __truediv__(self, fraction): # pour Python 3
    def __div__(self, fraction):
            if (isinstance(fraction,int) or isinstance(fraction,float)):
                fraction=Fractions(fraction)
            if isinstance(fraction,Fractions):
                return Fractions(self.n * fraction.d, self.d * fraction.n)
            else:
                return self*~fraction
    def __rdiv__(self,other):
        return other*~self
    def __invert__(self):
        return Fractions(self.d,self.n)
    def __neg__(self):
        return Fractions(-self.n,self.d)

    def __pow__(self,n):
        result=1
        for i in range(n):
            result=result*self
        return result
    def __trunc__(self):
        return self.n//self.d

    def __lt__(self, other):
        if isinstance(other,int) or isinstance(other,float):
            other=Fractions(other)
        if other.d*self.d > 0:
            return self.n*other.d < self.d * other.n
        else :
            return not(self.n*other.d < self.d * other.n)

    def __le__(self, other):
        if isinstance(other,int) or isinstance(other,float):
            other=Fractions(other)
        if other.d*self.d > 0:
            return self.n*other.d <= self.d * other.n
        else :
            return not(self.n*other.d <= self.d * other.n)

    def __eq__(self, other):
        if isinstance(other,int) or isinstance(other,float):
            other=Fractions(other)
        if isinstance(other,Fractions):
            return self.n*other.d == self.d * other.n

    def __ne__(self, other):
        if isinstance(other,int) or isinstance(other,float):
            other=Fractions(other)
        return self.n*other.d != self.d * other.n

    def __gt__(self, other):
        if isinstance(other,int) or isinstance(other,float):
            other=Fractions(other)
        if other.d*self.d > 0:
            return self.n*other.d > self.d * other.n
        else :
            return not(self.n*other.d > self.d * other.n)

    def __ge__(self, other):
        if isinstance(other,int) or isinstance(other,float):
            other=Fractions(other)
        if other.d*self.d > 0:
            return self.n*other.d >= self.d * other.n
        else :
            return not(self.n*other.d >= self.d * other.n)

    def __float__(self):
        return 1.0*self.n/self.d

    def __int__(self):
        try:
          assert (self.n / self.d == int(self.n / self.d)), 'La fraction n\'est pas un nombre entier !'
          return int(self.n)
        except AssertionError, args:
          print '%s: %s' % (args.__class__.__name__, args)





# Pyromaths
# -*- coding: utf-8 -*-
#
# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if notPopen, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

import random
from ..outils.Affichage import decimaux

def noms_sommets(nb):
    """Renvoie nb noms de sommets"""
    (listenb, listepts) = ([], [])
    for i in range(26):
        listenb.append(i + 65)
    for i in range(nb):
        listepts.append(str(chr(listenb.pop(random.randrange(26 - i)))))
    listepts.sort()
    return tuple(listepts)

def angle(coordo):
    """Angle pour placer le nom du point en fonction du quadrant"""
    l=[]
    for coord in coordo:
        if coord[0]>=0 and coord[1]>=0:
            ang='45'
        if coord[0]>=0 and coord[1]<0:
            ang='-45'
        if coord[0]<0 and coord[1]<0:
            ang='-135'
        if coord[0]<0 and coord[1]>=0:
            ang='135'


        l.append(ang)
    return l

def place_points(coordo,points):
    """coordo: liste de coordonnées ;
    points : noms des points associés
    génère le code latex pour placer tous les points"""
    l= angle(coordo)
    text="\\pstGeonode[PointSymbol=x,PosAngle={"
    for ang in l :
        text=text+ang+','
    text=text+"},PointNameSep=0.4]"
    i=0
    while i<len(coordo):
        text=text+str(coordo[i])+"{"+str(points[i])+"}"
        i=i+1
    return text

def valide(coord,liste_coord):
    """Évite deux points trop proches l'un de l'autre"""
    rep=True
    for c in liste_coord:
        if abs(coord[0]-c[0])<=0.5 and abs(coord[1]-c[1])<=0.5:
            rep=False
            break
    return rep

def quadrant(coord,liste_coord):
    """Évite 2 points dans le même quadrant"""
    rep= True
    for c in liste_coord:
        if coord[0]*c[0]>=0 and coord[1]*c[1]>=0:
            rep=False
            break
    return rep


def coordo_pts(nb):
    """Génère une liste de nb coordonnées. Je m'arrange pour avoir des points
    sur les 4 quadrants et sur les deux axes"""
    j=0
    k=0
    l=[]
    i=0
    while i<4 and i<nb:
        j=j+1
        if j==600:
            # Correction d'un bug
            l, i, j = [], 0, 0
        a=float(random.randrange(-9,10))/2
        b=float(random.randrange(-9,10))/2
        if ((a,b) not in l) and valide((a,b),l)and quadrant((a,b),l):
            l.append((a,b))
            i=i+1
    if nb>=4:
        a=float(random.randrange(-9,10))/2
        while not valide((0,a),l):
            a=float(random.randrange(-9,10))/2
        rg=random.randrange(0,len(l)-1)
        l[rg:rg]=[(0,a)]
        i=i+1

    if nb>=5:
        b=float(random.randrange(-9,10))/2
        while not valide((b,0),l):
            b=float(random.randrange(-9,10))/2
        rg=random.randrange(0,len(l)-1)
        l[rg:rg]=[(b,0)]
        i=i+1


    while i>=6 and i<nb and i<10:
        a=float(random.randrange(-9,10))/2
        b=float(random.randrange(-9,10))/2
        j=j+1
        if j==600:
            break
        if ((a,b) not in l) and valide((a,b),l)and quadrant((a,b),l[6:i]):
            l.append((a,b))
            i=i+1
    if nb>=11 and len(l)==10:
        a=float(random.randrange(-9,10))/2
        while not valide((0,a),l):
            a=float(random.randrange(-9,10))/2
        rg=random.randrange(6,len(l)-1)
        l[rg:rg]=[(0,a)]
        i=i+1
    if nb>=11 and len(l)==11:
        b=float(random.randrange(-9,10))/2
        while not valide((b,0),l):
            b=float(random.randrange(-9,10))/2
        rg=random.randrange(6,len(l)-1)
        l[rg:rg]=[(b,0)]
        i=i+1

    while i>=12 and i<nb :
        a=float(random.randrange(-9,10))/2
        b=float(random.randrange(-9,10))/2
        k=k+1
        if k==600:
            break
        if ((a,b) not in l) and valide((a,b),l):
            l.append((a,b))
            i=i+1
    return l


def affiche_coord(coord):
    """Affiche les coordonnées des points au format LaTeX"""
    return '\\hbox{$('+decimaux(str(coord[0]), 1) + '~;~' + \
                                             decimaux(str(coord[1]), 1)+')$}'

def tex_liste_co(liste_coord):
    """prend une liste de coordonnées et renvoie la liste en string prête pour latex
    #\hbox évite que les coordonnées soient coupées à l'affichage"""
    i=0
    tlc=[]
    while i < len(liste_coord):
        tlc.append(affiche_coord(liste_coord[i]))
        i=i+1
    return tlc

def main():
    nbpts = 13
    noms_pts=(noms_sommets(nbpts))
    coord_pts=coordo_pts(nbpts)
    voc=['abscisse',u'ordonnée']
    rg1=random.randrange(0,2)
    rg2=abs(rg1-1)
    while len(coord_pts)<nbpts:
        coord_pts=coordo_pts(nbpts)
    exo=["\\exercice",
         "\\parbox{0.4\\linewidth}{",
         "\\begin{enumerate}",
         u"\\item Donner les coordonnées des points %s, %s, %s, %s, %s et %s." % tuple(noms_pts[0:6]),
         u"\\item Placer dans le repère les points %s, %s, %s, %s, %s et %s" %noms_pts[6:12] +u" de coordonnées respectives %s, %s, %s, %s, %s et %s. " %tuple(tex_liste_co(coord_pts[6:12])),
         u"\\item Placer dans le repère le point %s d'%s %s et d'%s %s" %(noms_pts[12],voc[rg1],decimaux(str(coord_pts[12][rg1])),voc[rg2],decimaux(str(coord_pts[12][rg2]))),
         "\\end{enumerate}}\\hfill ",
         "\\parbox{0.55\\linewidth}{",
         "\\psset{unit=0.8cm}",
         "\\begin{pspicture}(-4.95,-4.95)(4.95,4.95)",
         "\\psgrid[subgriddiv=2, subgridcolor=lightgray, gridlabels=8pt](0,0)(-5,-5)(5,5)",
         "\\psline[linewidth=1.2pt]{->}(-5,0)(5,0)",
         "\\psline[linewidth=1.2pt]{->}(0,-5)(0,5)",
         place_points(coord_pts[0:6],noms_pts[0:6]),
         "\\end{pspicture}}"]
    cor=["\\exercice*",
         "\\parbox{0.4\\linewidth}{",
         "\\psset{unit=0.8cm}",
         "\\begin{pspicture}(-5,-5)(5,5)",
         "\\psgrid[subgriddiv=2, subgridcolor=lightgray, gridlabels=8pt](0,0)(-5,-5)(5,5)",
         "\\psline[linewidth=1.2pt]{->}(-5,0)(5,0)",
         "\\psline[linewidth=1.2pt]{->}(0,-5)(0,5)",
         place_points(coord_pts[0:13],noms_pts[0:13]),
         "\\end{pspicture}}\\hfill",
         "\\parbox{0.5\\linewidth}{",
         "\\begin{enumerate}",
         u"\\item Donner les coordonnées des points %s, %s, %s, %s, %s et %s." % tuple(noms_pts[0:6])]
    i=0
    while i<6:
        cor.append(u"Les coordonnées du point %s sont %s \n"%(noms_pts[i],affiche_coord(coord_pts[i])))
        i=i+1
    cor[len(cor):len(cor)]=[u"\\item Placer dans le repère les points %s, %s, %s, %s, %s et %s" %noms_pts[6:12] +u" de coordonnées respectives %s, %s, %s, %s, %s et %s. " %tuple(tex_liste_co(coord_pts[6:12])),
         u"\\item Placer dans le repère le point %s d'%s %s et d'%s %s" %(noms_pts[12],voc[rg1],decimaux(str(coord_pts[12][rg1])),voc[rg2],decimaux(str(coord_pts[12][rg2]))),
         "\\end{enumerate}"]
    cor.append("}")
    return(exo,cor)




#!/usr/bin/python
# -*- coding: utf-8 -*-

from ..classes.Fractions import Fractions
from . import Arithmetique, Affichage
import random
import math


#===============================================================================
# Gère le calcul fractionnaire
#===============================================================================


def EffectueSommeFractions(fr1, fr2, s, pre, post):
    if s == "+":
        fr = fr1 + fr2
    else:
        fr = fr1 - fr2
    cor = []
    if fr1.n and fr2.n:
        ppcm = Arithmetique.ppcm(fr2.d, fr1.d)
        if abs(fr1.d) - abs(fr2.d):
            cor.append("%s%s%s%s%s" % (pre, Fractions.TeX(fr1, True,
                       coef=ppcm // abs(fr1.d)), s, Fractions.TeX(fr2,
                       True, coef=ppcm // abs(fr2.d)), post))
        if pre.rstrip().endswith('(') and post.lstrip().startswith(')'):
            pre = pre.rstrip()[:len(pre.rstrip()) - 1]
            post = post.lstrip()[1:]
        cor.append("%s%s%s" % (pre, Fractions.TeX(fr, True), post))
        if fr.n:
            frs = Fractions.simplifie(fr)
            if abs(frs.n) != abs(fr.n):
                cor.append("%s%s%s" % (pre, Fractions.TeX(frs, True,
                           coef=fr.d // frs.d), post))
                cor.append("%s%s%s" % (pre, Fractions.TeX(frs, True),
                           post))
    else:
        cor.append("%s%s%s" % (pre, Fractions.TeX(fr, True), post))
    return cor


def EffectueProduitFractions(fr1, fr2, pre, post):
    fr1s = Fractions.simplifie(fr1)
    fr2s = Fractions.simplifie(fr2)
    cor = []
    if fr1.n and fr2.n:
        if abs(fr1s.d) < abs(fr1.d) or abs(fr2s.d) < abs(fr2.d):
            cor.append("%s%s \\times %s%s" % (pre, Fractions.TeXSimplifie(fr1),
                       Fractions.TeXSimplifie(fr2), post))
            cor.append("%s%s \\times %s%s" % (pre, Fractions.TeX(fr1s,
                       True), Fractions.TeX(fr2s, True), post))
        fr = fr1s * fr2s
        frs = Fractions.simplifie(fr)
        if abs(frs.d - fr.d):
            cor.append("%s%s%s" % (pre, Fractions.TeXProduit(fr1s, fr2s),
                       post))
        cor.append("%s%s%s" % (pre, Fractions.TeX(frs, True), post))
    else:
        cor.append("%s%s%s" % (pre, Fractions.TeX(Fractions(0), True),
                   post))
    return cor


def EffectueQuotientFractions(fr1, fr2, pre, post):
    fr2 = Fractions(1, 1) / fr2
    cor = ["%s%s \\times %s%s" % (pre, Fractions.TeX(fr1, True),
           Fractions.TeX(fr2, True), post)]
    cor.extend(EffectueProduitFractions(fr1, fr2, pre, post))
    return cor




#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#

from .fractions import tex_frac, simplifie, tex_decomp_prod, decomp_prod, \
    produit, den_com0
from ..outils.Arithmetique import valeur_alea, signe, pgcd
import random

#
# ------------------- DEVELOPPEMENTS -------------------


def coef_opposes(a):  # renvoie un tuple dont les valeurs sont les opposees de celles de a
    l = []
    for i in range(len(a)):
        l.append(-a[i])
    return tuple(l)


def dev(a):  # renvoi un tuple avec les 3 coefficients du developpement
    return (a[0][0] * a[1][0], a[0][0] * a[1][1] + a[0][1] * a[1][0], a[0][1] *
            a[1][1])


def somme_polynomes(a, b):  # renvoie un tuple dont les valeurs sont les sommes des valeurs correspondantes dans a et b
    l = []
    if len(a) > len(b):
        long = len(a)
    else:
        long = len(b)
    for i in range(long):
        if (len(a) - i) - 1 < 0:
            l.append(b[(len(b) - 1) - i])
        elif (len(b) - i) - 1 < 0:
            l.append(a[(len(a) - 1) - i])
        else:
            l.append(a[(len(a) - 1) - i] + b[(len(b) - 1) - i])
    l.reverse()
    return tuple(l)


def tex_coef(coef, var, bplus=0, bpn=0, bpc=0):

    # coef est le coefficient à écrire devant la variable var
    # bplus est un booleen : s'il est vrai, il faut ecrire le signe +
    # bpn est un booleen : s'il est vrai, il faut mettre des parentheses autour de l'ecriture si coef est negatif.
    # bpc est un booleen : s'il est vrai, il faut mettre des parentheses autour de l'ecriture si coef =! 0 ou 1 et var est non vide

    if coef != 0 and abs(coef) != 1:
        if var == '':
            if abs(coef) >= 1000:
                a = '\\nombre{%s}' % coef
            else:
                a = '%s' % coef
        else:
            if abs(coef) >= 1000:
                a = '\\nombre{%s}\\,%s' % (coef, var)
            else:
                a = '%s\\,%s' % (coef, var)
        if bplus and coef > 0:
            a = '+' + a
    elif coef == 1:
        if var == '':
            a = '1'
        else:
            a = '%s' % var
        if bplus:
            a = '+' + a
    elif coef == 0:
        a = ''
    elif coef == -1:
        if var == '':
            a = '-1'
        else:
            a = '-%s' % var
    if bpn and coef < 0 or bpc and coef != 0 and coef != 1 and var != '':
        a = '\\left( ' + a + '\\right)'
    return a


def tex_dev0(a, bplus=0):  # renvoi (a+b)², (a-b)² ou (a+b)(c+d) ou a(c+d) ou (a+b)*c

    # a est de la forme ((3, 2), (3, 2)) pour (3x+2)(3x+2)

    (ca, cb, cc, cd) = (a[0][0], a[0][1], a[1][0], a[1][1])  # coefficients a, b, c et d
    if ca == 0 and cb == 0 or cc == 0 and cd == 0:
        return '0'
    elif a[0] == a[1]:

                        # (a+b)² ou (a-b)²

        if ca == 0:
            return '%s^2' % tex_coef(cb, '', bpn=1)
        elif cb == 0:
            return '%s^2' % tex_coef(ca, 'x', bpc=1)
        else:
            return '(%s%s)^2' % (tex_coef(ca, 'x'), tex_coef(cb, '',
                                 bplus=1))
    else:

        # (a+b)(c+d)

        if ca == 0 or cb == 0:
            if cc == 0 or cd == 0:
                return '%s%s\\times %s%s' % (tex_coef(ca, 'x'), tex_coef(cb,
                        '', bplus=ca != 0), tex_coef(cc, 'x', bpn=1),
                        tex_coef(cd, '', bplus=cc != 0, bpn=1))
            else:
                return '%s%s\\,(%s%s)' % (tex_coef(ca, 'x'), tex_coef(cb,
                        '', bplus=ca != 0), tex_coef(cc, 'x'), tex_coef(cd,
                        '', bplus=cc != 0))
        elif cc == 0 or cd == 0:
            if cc == 0 and cd == 1:
                return '%s%s' % (tex_coef(ca, 'x', bplus=bplus),
                                 tex_coef(cb, '', bplus=ca != 0))
            else:
                return '(%s%s)\\times %s%s' % (tex_coef(ca, 'x'),
                        tex_coef(cb, '', bplus=ca != 0), tex_coef(cc,
                        'x', bpn=1), tex_coef(cd, '', bplus=cc != 0, bpn=
                        1))
        else:
            return '(%s%s)\\,(%s%s)' % (tex_coef(ca, 'x'), tex_coef(cb,
                    '', bplus=1), tex_coef(cc, 'x'), tex_coef(cd, '',
                    bplus=1))


def tex_dev1(a, bplus=0, bpar=0, bpn=0):  # renvoi le developpement (a)²+2*a*b+(b)², (a)²-2*a*b+(b)², (a)²-(b)² ou a*c+a*d+b*c+b*d

    # a est de la forme ((3, 2)(3, 2)) pour (3x+2)(3x+2)

    (ca, cb, cc, cd) = (a[0][0], a[0][1], a[1][0], a[1][1])  # coefficients a, b, c et d
    if a[0] == a[1]:  # (a+b)² ou (a-b)²
        if signe(ca) == signe(cb):  # (a+b)²
            (ca, cb) = (abs(ca), abs(cb))
            texte = '%s^2+2\\times %s\\times %s+%s^2' % (tex_coef(ca,
                    'x', bpc=1), tex_coef(ca, 'x', bpn=1), tex_coef(cb,
                    '', bpn=1), tex_coef(cb, '', bpn=1, bpc=1))
            if bpar:
                return '(' + texte + ')'
            else:
                return texte
        else:

            # (a-b)²

            (ca, cb) = (abs(ca), abs(cb))
            texte = '%s^2-2\\times %s\\times %s+%s^2' % (tex_coef(ca,
                    'x', bpc=1), tex_coef(ca, 'x', bpn=1), tex_coef(cb,
                    '', bpn=1), tex_coef(cb, '', bpn=1, bpc=1))
            if bpar:
                return '(' + texte + ')'
            else:
                return texte
    if abs(ca) == abs(cc) and abs(cb) == abs(cd):  # (a+b)(a-b) ou (a+b)(-a+b)
        if ca == cc:  # (a+b)(a-b)
            texte = '%s^2-%s^2' % (tex_coef(ca, 'x', bpc=1), tex_coef(abs(cb),
                                   ''))
            if bpar:
                return '(' + texte + ')'
            else:
                return texte
        else:

            # (a+b)(-a+b)

            texte = '%s^2-%s^2' % (tex_coef(cb, '', bpn=1), tex_coef(abs(ca),
                                   'x', bpc=1))
            if bpar:
                return '(' + texte + ')'
            else:
                return texte
    else:

        # (a+b)(c+d)

        if cc == 0 and cd == 1:
            texte = '%s%s' % (tex_coef(ca, 'x', bplus=bplus), tex_coef(cb,
                              '', bplus=ca != 0))
            if bpar:
                return '(' + texte + ')'
            else:
                return texte
        else:
            texte = '%s+%s+%s+%s' % (tex_coef(ca * cc, 'x^2', bpn=bpn),
                    tex_coef(ca * cd, 'x', bpn=1), tex_coef(cb * cc, 'x',
                    bpn=1), tex_coef(cb * cd, '', bpn=1))
            if bpar:
                return '(' + texte + ')'
            else:
                return texte


def tex_developpe1(a, exo, cor):  # developpe l'expression a
    exo.append(u'\\[ \\thenocalcul = ' + tex_dev0(a) + '\\]')
    cor.append(u'\\[ \\thenocalcul = ' + tex_dev0(a) + '\\]')
    cor.append(u'\\[ \\thenocalcul = ' + tex_dev1(a) + '\\]')
    cor.append(u'\\[ \\boxed{\\thenocalcul = ' + tex_trinome(dev(a)) + '} \\]')


def tex_developpe2(a, nega, b, negb, cor, exo=None):  # developpe l'expression a plus l'expression b.
    if nega:
        if negb:
            if b[1][0] == 0 and b[1][1] == 1:
                if exo:
                    exo.append(u'\\[ \\thenocalcul = ' + '-' + tex_dev0(a) + '-(' + tex_dev0(b) +
                              ')' + '\\]')
                cor.append(u'\\[ \\thenocalcul = ' + '-' + tex_dev0(a) + '-(' + tex_dev0(b) +
                          ')' + '\\]')
            elif a[1][0] == 0 and a[1][1] == 1:
                if exo:
                    exo.append(u'\\[ \\thenocalcul = ' + '-(' + tex_dev0(a) + ')-' + tex_dev0(b) + '\\]')
                cor.append(u'\\[ \\thenocalcul = ' + '-(' + tex_dev0(a) + ')-' + tex_dev0(b) + '\\]')
            else:
                if exo:
                    exo.append(u'\\[ \\thenocalcul = ' + '-' + tex_dev0(a) + '-' + tex_dev0(b) + '\\]')
                cor.append(u'\\[ \\thenocalcul = ' + '-' + tex_dev0(a) + '-' + tex_dev0(b) + '\\]')
            cor.append(u'\\[ \\thenocalcul = ' + '-(' + tex_dev1(a) + ')-(' + tex_dev1(b) + ')' + '\\]')
            cor.append(u'\\[ \\thenocalcul = ' + '-(' + tex_trinome(dev(a)) + ')-(' +
                      tex_trinome(dev(b)) + ')' + '\\]')
            cor.append(u'\\[ \\thenocalcul = ' + tex_trinome(coef_opposes(dev(a))) +
                      tex_trinome(coef_opposes(dev(b)), bplus=1) + '\\]')
            cor.append(u'\\[ \\boxed{\\thenocalcul = ' + tex_trinome(somme_polynomes(coef_opposes(dev(a)),
                      coef_opposes(dev(b)))) + '} \\]')
        else:
            if a[1][0] == 0 and a[1][1] == 1:
                if exo:
                    exo.append(u'\\[ \\thenocalcul = ' + '-(' + tex_dev0(a) + ')+' + tex_dev0(b) + '\\]')
                cor.append(u'\\[ \\thenocalcul = ' + '-(' + tex_dev0(a) + ')+' + tex_dev0(b) + '\\]')
                cor.append(u'\\[ \\thenocalcul = ' + '-(' + tex_dev1(a) + ')+' + tex_dev1(b,
                          bpn=1) + '\\]')
                cor.append(u'\\[ \\thenocalcul = ' + '-(' + tex_trinome(dev(a)) + ')+' +
                          tex_trinome(dev(b)) + '\\]')
            elif b[1][0] == 0 and b[1][1] == 1:
                if exo:
                    exo.append(u'\\[ \\thenocalcul = ' + '-' + tex_dev0(a) + tex_dev0(b, bplus=
                              1) + '\\]')
                cor.append(u'\\[ \\thenocalcul = ' + '-' + tex_dev0(a) + tex_dev0(b, bplus=1) + '\\]')
                cor.append(u'\\[ \\thenocalcul = ' + '-(' + tex_dev1(a) + ')' + tex_dev1(b,
                          bplus=1) + '\\]')
                cor.append(u'\\[ \\thenocalcul = ' + '-(' + tex_trinome(dev(a)) + ')' +
                          tex_trinome(dev(b), bplus=1) + '\\]')
            else:
                if exo:
                    exo.append(u'\\[ \\thenocalcul = ' + '-' + tex_dev0(a) + '+' + tex_dev0(b) + '\\]')
                cor.append(u'\\[ \\thenocalcul = ' + '-' + tex_dev0(a) + '+' + tex_dev0(b) + '\\]')
                cor.append(u'\\[ \\thenocalcul = ' + '-(' + tex_dev1(a) + ')+' + tex_dev1(b,
                          bpn=1) + '\\]')
                cor.append(u'\\[ \\thenocalcul = ' + '-(' + tex_trinome(dev(a)) + ')' +
                          tex_trinome(dev(b), bplus=1) + '\\]')
            cor.append(u'\\[ \\thenocalcul = ' + tex_trinome(coef_opposes(dev(a))) +
                      tex_trinome(dev(b), bplus=1) + '\\]')
            cor.append(u'\\[ \\boxed{\\thenocalcul = ' + tex_trinome(somme_polynomes(coef_opposes(dev(a)),
                      dev(b))) + '} \\]')
    elif negb:
        if b[1][0] == 0 and b[1][1] == 1:
            if exo:
                exo.append(u'\\[ \\thenocalcul = ' + tex_dev0(a) + '-(' + tex_dev0(b) + ')' + '\\]')
            cor.append(u'\\[ \\thenocalcul = ' + tex_dev0(a) + '-(' + tex_dev0(b) + ')' + '\\]')
        else:
            if exo:
                exo.append(u'\\[ \\thenocalcul = ' + tex_dev0(a) + '-' + tex_dev0(b) + '\\]')
            cor.append(u'\\[ \\thenocalcul = ' + tex_dev0(a) + '-' + tex_dev0(b) + '\\]')
        cor.append(u'\\[ \\thenocalcul = ' + tex_dev1(a) + '-(' + tex_dev1(b) + ')' + '\\]')
        cor.append(u'\\[ \\thenocalcul = ' + tex_trinome(dev(a)) + '-(' + tex_trinome(dev(b)) +
                  ')' + '\\]')
        cor.append(u'\\[ \\thenocalcul = ' + tex_trinome(dev(a)) + tex_trinome(coef_opposes(dev(b)),
                  bplus=1) + '\\]')
        cor.append(u'\\[ \\boxed{\\thenocalcul = ' + tex_trinome(somme_polynomes(dev(a), coef_opposes(dev(b)))) + '} \\]')
    else:
        if b[1][0] == 0 and b[1][1] == 1:
            if exo:
                exo.append(u'\\[ \\thenocalcul = ' + tex_dev0(a) + tex_dev0(b, bplus=1) + '\\]')
            cor.append(u'\\[ \\thenocalcul = ' + tex_dev0(a) + tex_dev0(b, bplus=1) + '\\]')
            cor.append(u'\\[ \\thenocalcul = ' + tex_dev1(a) + tex_dev1(b, bplus=1) + '\\]')
        else:
            if exo:
                exo.append(u'\\[ \\thenocalcul = ' + tex_dev0(a) + '+' + tex_dev0(b) + '\\]')
            cor.append(u'\\[ \\thenocalcul = ' + tex_dev0(a) + '+' + tex_dev0(b) + '\\]')
            cor.append(u'\\[ \\thenocalcul = ' + tex_dev1(a) + '+' + tex_dev1(b) + '\\]')
        cor.append(u'\\[ \\thenocalcul = ' + tex_trinome(dev(a)) + tex_trinome(dev(b), bplus=1) + '\\]')
        cor.append(u'\\[ \\boxed{\\thenocalcul = ' + tex_trinome(somme_polynomes(dev(a), dev(b))) + '} \\]')


def tex_trinome(a, bplus=0, bpar=0):  # renvoi le trinome ax²+bx+c

    # bplus est un booleen : s'il est vrai, le premier terme est precede d'un signe + si son coefficient est positif.
    # bpar est un booleen : s'il est vrai, ecrit des parentheses autour du trinome si deux valeurs au moins ne sont pas nulles ou si la valeur est -

    texte = '%s%s%s' % (tex_coef(a[0], 'x^2', bplus=bplus), tex_coef(a[1],
                        'x', bplus=bplus or a[0] != 0), tex_coef(a[2],
                        '', bplus=bplus or a[0] != 0 or a[1] != 0))
    if bpar:
        v0 = 0
        for i in range(3):  # compte le nombre de valeurs nulles
            if a[i] == 0:
                v0 = v0 + 1
        if v0 == 2:  # une seule valeur non nulle (on suppose que les trois coef ne sont pas nuls)
            if a[0] < 0 or a[1] < 0 or a[2] < 0:  # le coefficient non nul est negatif, il faut donc des parentheses
                return '(' + texte + ')'
            else:
                return texte
        else:
            return '(' + texte + ')'
    else:
        return texte


def valeurs_apb2(pyromax):  # renvoie un tuple contenant ((3,5),(3,5))
    a = valeur_alea(1, pyromax)
    b = valeur_alea(1, pyromax)
    return ((a, b), (a, b))


def valeurs_amb2(pyromax):  # renvoie un tuple contenant ((3,-5),(3,-5))
    a = valeur_alea(1, pyromax)
    b = valeur_alea(-pyromax, -1)
    return ((a, b), (a, b))


def valeurs_apbamb(pyromax):  # renvoie un tuple contenant ((3,-5),(3,+5))
    a = valeur_alea(1, pyromax)
    b = valeur_alea(1, pyromax)
    if random.randrange(2):
        return ((a, -b), (a, b))
    else:
        return ((a, b), (a, -b))


def valeurs_distr(pyromax):

    # renvoie in tuple contenant ((a,b),(c,d)) avec a != c ou b != d (en valeur
    # absolue)

    a = valeur_alea(-pyromax, pyromax)
    b = valeur_alea(-pyromax, pyromax)
    c = valeur_alea(-pyromax, pyromax)
    d = valeur_alea(-pyromax, pyromax)
    while abs(a) == abs(c) and abs(b) == abs(d):
        c = valeur_alea(-pyromax, pyromax)
        d = valeur_alea(-pyromax, pyromax)
    return ((a, b), (c, d))


#
# ------------------- FACTORISATIONS -------------------


def facteur_commun(a, b):  #renvoie le facteur commun et les les deux autres binomes
    if a[0] == b[0]:  # recherche le facteur commun et le nomme f0, f1 et f2 etant les deux autres facteurs
        (f0, f1, f2) = (a[0], a[1], b[1])
    elif a[0] == b[1]:
        (f0, f1, f2) = (a[0], a[1], b[0])
    elif a[1] == b[0]:
        (f0, f1, f2) = (a[1], a[0], b[1])
    elif a[1] == b[1]:
        (f0, f1, f2) = (a[1], a[0], b[0])
    return (f0, f1, f2)


def facteur_commun2(f, nega, negb):  # renvoie (2x+3)(4x+5+6x+7) sous la forme ((2,3),(4,5),(6,7))
    if nega:
        if negb:
            return (f[0], coef_opposes(f[1]), coef_opposes(f[2]))
        else:
            return (f[0], coef_opposes(f[1]), f[2])
    elif negb:
        return (f[0], f[1], coef_opposes(f[2]))
    else:
        return (f[0], f[1], f[2])


def facteur_commun3(f, nega, negb):  # renvoie un tuple contenant les deux binomes finaux de la mise en facteur
    if nega and negb:
        return (f[0], somme_polynomes(coef_opposes(f[1]), coef_opposes(f[2])))
    elif nega:
        return (f[0], somme_polynomes(coef_opposes(f[1]), f[2]))
    elif negb:
        return (f[0], somme_polynomes(f[1], coef_opposes(f[2])))
    else:
        return (f[0], somme_polynomes(f[1], f[2]))


def tex_binome(a, bplus=0, bpar=0):  # renvoi le binome ax+b

    # bplus est un booleen : s'il est vrai, le premier terme est precede d'un signe + si son coefficient est positif.
    # bpar est un booleen : s'il est vrai, ecrit des parentheses autour du binome si les deux coef ne sont pas nuls ou si le seul coef est -

    texte = '%s%s' % (tex_coef(a[0], 'x', bplus=bplus), tex_coef(a[1],
                      '', bplus=bplus or a[0] != 0))
    if bpar:
        if (a[0] != 0 or a[1] < 0) and (a[1] != 0 or a[0] < 0):
            texte = '(' + texte + ')'
    return texte


def tex_trinome_alea(a, bplus=0):  # renvoi le trinome ax²+bx+c dans le desordre
    valeurs = [tex_coef(a[0], 'x^2', bplus=1), tex_coef(a[1], 'x', bplus=
               1), tex_coef(a[2], '', bplus=1)]
    ordre1 = []
    while ordre1 == [] or ordre1 == [0, 1, 2]:
        ordre0 = [i for i in range(3)]
        ordre1 = [ordre0.pop(random.randrange(3 - i)) for i in range(3)]
    liste = [valeurs[ordre1[i]] for i in range(3)]
    if not bplus and liste[0].startswith('+'):
        liste[0] = liste[0].replace('+', '', 1)
    return '%s%s%s' % tuple(liste)


def tex_facteur_commun0(a):  #renvoie -(2x+3)(x+5)+(2x+3)(3x+6)
    if a[2]:
        if a[3]:
            return '-' + tex_dev0(a[0]) + '-' + tex_dev0(a[1])
        else:
            return '-' + tex_dev0(a[0]) + '+' + tex_dev0(a[1])
    elif a[3]:
        return tex_dev0(a[0]) + '-' + tex_dev0(a[1])
    else:
        return tex_dev0(a[0]) + '+' + tex_dev0(a[1])


def tex_facteur_commun1(f, nega, negb):  # renvoie (2x+3)(-(x+5)+3x+6)
    texte = tex_binome(f[0], bpar=1)
    if nega or negb:
        texte = texte + '\\,\\big( '
    else:
        texte = texte + '\\,('
    if nega:
        texte = texte + '-' + tex_binome(f[1], bpar=1)
    else:
        texte = texte + tex_binome(f[1], bpar=0)
    if negb:
        texte = texte + '-' + tex_binome(f[2], bpar=1)
    else:
        texte = texte + tex_binome(f[2], bplus=1)
    if nega or negb:
        texte = texte + '\\big)'
    else:
        texte = texte + ')'
    return texte


def tex_facteur_commun2(a):  # renvoie renvoie (2x+3)(-x-5+3x+6)
    return tex_binome(a[0], bpar=1) + '\\,(' + tex_binome(a[1]) + \
        tex_binome(a[2], bplus=1) + ')'


def tex_a2moinsb2_0(a):  # renvoie 16-(2x+3)²
    if a[0][0] == 0:
        return '%s-%s^2' % (a[0][1] ** 2, tex_binome(a[1], bpar=1))
    else:
        return '%s^2-%s' % (tex_binome(a[0], bpar=1), a[1][1] ** 2)


def tex_a2moinsb2_2(a):  # renvoie (4+2x+3)(4-(2x+3))
    if a[0][0] == 0:
        return '(%s%s)\\,\\big( %s-%s\\big)' % (tex_binome(a[0]),
                tex_binome(a[1], bplus=1), tex_binome(a[0]), tex_binome(a[1],
                bpar=1))
    else:
        return '(%s%s)\\,(%s-%s)' % (tex_binome(a[0]), tex_binome(a[1],
                bplus=1), tex_binome(a[0]), tex_binome(a[1], bpar=1))


def tex_a2moinsb2_3(a):  # renvoie (4+2x+3)(4-2x-3)
    if a[0][0] == 0:
        return '(%s%s)\\,(%s%s)' % (tex_binome(a[0]), tex_binome(a[1],
                                    bplus=1), tex_binome(a[0]),
                                    tex_binome(coef_opposes(a[1]), bplus=
                                    1))
    else:
        return ''


def tex_facteur_commun(valeurs, cor, exo=None):  # ecrit toutes les etapes de la factorisation
    (a, b, nega, negb) = valeurs
    f = facteur_commun(a, b)
    if exo:
        exo.append(u'\\[ \\thenocalcul = ' + tex_facteur_commun0(valeurs) + '\\]')
    cor.append(u'\\[ \\thenocalcul = ' + tex_facteur_commun0(valeurs) + '\\]')
    cor.append(u'\\[ \\thenocalcul = ' + tex_facteur_commun1(f, nega, negb) + '\\]')
    f2 = facteur_commun2(f, nega, negb)
    if nega and f2[1][0] != 0 and f2[1][1] != 0 or negb and f2[2][0] != \
        0 and f2[2][0] != 0:
        cor.append(u'\\[ \\thenocalcul = ' + tex_facteur_commun2(f2) + '\\]')
    cor.append(u'\\[ \\boxed{\\thenocalcul = ' + tex_dev0((f2[0], somme_polynomes(f2[1], f2[2]))) + '} \\]')


def tex_a2moinsb2(valeurs, cor, exo):  # ecrit toutes les etapes de la factorisation
    if exo:
        exo.append(u'\\[ \\thenocalcul = ' + tex_a2moinsb2_0(valeurs) + '\\]')
    cor.append(u'\\[ \\thenocalcul = ' + tex_a2moinsb2_0(valeurs) + '\\]')
    cor.append(u'\\[ \\thenocalcul = ' + '%s^2-%s^2' % (tex_binome(valeurs[0], bpar=1),
              tex_binome(valeurs[1], bpar=1)) + '\\]')
    cor.append(u'\\[ \\thenocalcul = ' + tex_a2moinsb2_2(valeurs) + '\\]')
    cor.append(u'\\[ \\thenocalcul = ' + tex_a2moinsb2_3(valeurs) + '\\]')
    cor.append(u'\\[ \\boxed{\\thenocalcul = ' + tex_dev0((somme_polynomes(valeurs[0], valeurs[1]),
              somme_polynomes(valeurs[0], coef_opposes(valeurs[1])))) + '} \\]')


def valeurs_facteur_commun(pyromax):  # renvoie les valeurs pour obtenir un facteur commun.
    a = ((valeur_alea(-pyromax, pyromax), valeur_alea(-pyromax, pyromax)),
         (valeur_alea(-pyromax, pyromax), valeur_alea(-pyromax, pyromax)))
    while a[0] == a[1]:  # on refuse un carre
        a = ((valeur_alea(-pyromax, pyromax), valeur_alea(-pyromax,
             pyromax)), (valeur_alea(-pyromax, pyromax), valeur_alea(-pyromax,
             pyromax)))
    if random.randrange(2) == 0:
        while True:
            b = (a[random.randrange(2)], (valeur_alea(-pyromax, pyromax),
                 valeur_alea(-pyromax, pyromax)))
            if b[0] != b[1]:
                break
    else:
        while True:
            b = ((valeur_alea(-pyromax, pyromax), valeur_alea(-pyromax,
                 pyromax)), a[random.randrange(2)])
            if b[0] != b[1]:
                break
    (nega, negb) = (random.randrange(2), random.randrange(2))
    return (a, b, nega, negb)


def valeurs_facteur1(pyromax):  # renvoie les valeurs pour obtenir un facteur commun facteur de 1
    a = ((valeur_alea(-pyromax, pyromax), valeur_alea(-pyromax, pyromax)),
         (valeur_alea(-pyromax, pyromax), valeur_alea(-pyromax, pyromax)))
    while a[0] == a[1]:  # on refuse un carre
        a = ((valeur_alea(-pyromax, pyromax), valeur_alea(-pyromax,
             pyromax)), (valeur_alea(-pyromax, pyromax), valeur_alea(-pyromax,
             pyromax)))
    b = (a[random.randrange(2)], (0, 1))
    if random.randrange(2) == 1:  # le 1 est dans le deuxieme terme
        return (a, b, random.randrange(2), random.randrange(2))
    else:
        return (b, a, random.randrange(2), random.randrange(2))


def valeurs_facteurcarre(pyromax):  # renvoie les valeurs pour obtenir un facteur commun au carre
    a = ((valeur_alea(-pyromax, pyromax), valeur_alea(-pyromax, pyromax)),
         (valeur_alea(-pyromax, pyromax), valeur_alea(-pyromax, pyromax)))
    while a[0] == a[1]:  # on refuse un carre
        a = ((valeur_alea(-pyromax, pyromax), valeur_alea(-pyromax,
             pyromax)), (valeur_alea(-pyromax, pyromax), valeur_alea(-pyromax,
             pyromax)))
    if random.randrange(2) == 0:
        b = (a[0], a[0])
    else:
        b = (a[1], a[1])
    if random.randrange(2) == 0:  # le carre est en premier
        return (b, a, 0, random.randrange(2))
    else:
        return (a, b, random.randrange(2), 0)


def valeurs_facteura2mb2(pyromax):  # renvoie (((a,b),(a,-b)),((a,b),(c,d));nega,negb)
    a = ((valeur_alea(-pyromax, pyromax), valeur_alea(-pyromax, pyromax)),
         (valeur_alea(-pyromax, pyromax), valeur_alea(-pyromax, pyromax)))
    while a[0] == a[1]:  # on refuse un carre
        a = ((valeur_alea(-pyromax, pyromax), valeur_alea(-pyromax,
             pyromax)), (valeur_alea(-pyromax, pyromax), valeur_alea(-pyromax,
             pyromax)))
    if random.randrange(2) == 0:
        a = ((abs(a[0][0]), a[0][1]), a[1])
        b = (a[0], (a[0][0], -a[0][1]))
    else:
        a = (a[0], (abs(a[1][0]), a[1][1]))
        b = (a[1], (a[1][0], -a[1][1]))
    if random.randrange(2) == 0:  # le a²-b² est en premier
        return (b, a, 0, random.randrange(2))
    else:
        return (a, b, random.randrange(2), 0)


def valeurs_a2moinsb2(pyromax):  # renvoie les valeurs pour obtenir a²-(bx+c)²
    a = (0, valeur_alea(1, pyromax))
    b = (valeur_alea(-pyromax, pyromax), valeur_alea(-pyromax, pyromax))
    if random.randrange(2) == 0:  #  le nombre est en premier
        return (a, b)
    else:
        return (b, a)


def factorisation0(cor, exo=None, valeurs=None):  # factorise (ax+b)(cx+d)+(ax+b)(ex+f)
    if not valeurs:
        valeurs = valeurs_facteur_commun(10)
    tex_facteur_commun(valeurs, cor, exo)


def tex_factorisation1(valeurs):  # renvoie la premiere etape de la factorisation 1
    if valeurs[2] == 0:
        siga = ''
    else:
        siga = '-'
    if valeurs[3] == 0:
        sigb = '+'
    else:
        sigb = '-'
    if valeurs[0][1][0] == 0 and valeurs[0][1][1] == 1:  # le 1 est dans la premiere partie
        return '%s%s%s%s' % (siga, tex_trinome(dev(valeurs[0]), bpar=1),
                             sigb, tex_dev0(valeurs[1]))
    else:
        return '%s%s%s%s' % (siga, tex_dev0(valeurs[0]), sigb,
                             tex_trinome(dev(valeurs[1]), bpar=1))


def factorisation1(cor, exo=None, valeurs=None):  # factorise (ax+b)(cx+d)+(ax+b)
    if not valeurs:
        valeurs = valeurs_facteur1(10)
    if valeurs[2] == 0:
        siga = ''
    else:
        siga = '-'
    if valeurs[3] == 0:
        sigb = '+'
    else:
        sigb = '-'
    if valeurs[0][1][0] == 0 and valeurs[0][1][1] == 1:  # le 1 est dans la premiere partie
        texte = '%s%s%s%s' % (siga, tex_trinome(dev(valeurs[0]), bpar=1),
                              sigb, tex_dev0(valeurs[1]))
        cor.append(u'\\[ \\thenocalcul = ' + texte + '\\]')
        if exo:
            exo.append(u'\\[ \\thenocalcul = ' + texte + '\\]')
        texte = '%s%s\\times1%s%s' % (siga, tex_trinome(dev(valeurs[0]),
                bpar=1), sigb, tex_dev0(valeurs[1]))
        cor.append(u'\\[ \\thenocalcul = ' + texte + '\\]')
    else:
        texte = '%s%s%s%s' % (siga, tex_dev0(valeurs[0]), sigb,
                              tex_trinome(dev(valeurs[1]), bpar=1))
        cor.append(u'\\[ \\thenocalcul = ' + texte + '\\]')
        if exo:
            exo.append(u'\\[ \\thenocalcul = ' + texte + '\\]')
        texte = '%s%s%s%s\\times1' % (siga, tex_dev0(valeurs[0]), sigb,
                tex_trinome(dev(valeurs[1]), bpar=1))
        cor.append(u'\\[ \\thenocalcul = ' + texte + '\\]')
    f = facteur_commun(valeurs[0], valeurs[1])
    cor.append(u'\\[ \\thenocalcul = ' + tex_facteur_commun1(f, valeurs[2], valeurs[3]) + '\\]')
    f2 = facteur_commun2(f, valeurs[2], valeurs[3])
    if valeurs[2] and f2[1][0] != 0 and f2[1][1] != 0 or valeurs[3] and \
        f2[2][0] != 0 and f2[2][0] != 0:
        cor.append(u'\\[ \\thenocalcul = ' + tex_facteur_commun2(f2) + '\\]')
    cor.append(u'\\[ \\boxed{\\thenocalcul = ' + tex_dev0((f2[0], somme_polynomes(f2[1], f2[2]))) + '} \\]')


def factorisation2(cor, exo=None, valeurs=None):  # factorise (ax+b)(cx+d)+(ax+b)²
    if not valeurs:
        valeurs = valeurs_facteurcarre(10)
    tex_facteur_commun(valeurs, cor, exo)


def tex_factorisation3(valeurs):  # renvoie les deux premieres etapes de la factorisation 3
    if valeurs[2] == 0:
        nega = ''
    else:
        nega = '-'
    if valeurs[3] == 0:
        negb = '+'
    else:
        negb = '-'
    if valeurs[0][0][0] == valeurs[0][1][0] and valeurs[0][0][1] == -valeurs[0][1][1]:
        return ('%s%s%s%s' % (nega, tex_trinome(dev(valeurs[0])), negb,
                tex_dev0(valeurs[1])), '%s%s%s%s' % (nega, tex_dev1(valeurs[0]),
                negb, tex_dev0(valeurs[1])))
    else:
        return ('%s%s%s%s' % (nega, tex_dev0(valeurs[0]), negb,
                tex_trinome(dev(valeurs[1]))), '%s%s%s%s' % (nega,
                tex_dev0(valeurs[0]), negb, tex_dev1(valeurs[1])))


def factorisation3(cor, exo=None, valeurs=None):  # factorise (ax+b)(cx+d)+(ax)²-b²
    if not valeurs:
        valeurs = valeurs_facteura2mb2(10)
    textes = tex_factorisation3(valeurs)
    if exo:
        exo.append(u'\\[ \\thenocalcul = ' + textes[0] + '\\]')
    cor.append(u'\\[ \\thenocalcul = ' + textes[0] + '\\]')
    cor.append(u'\\[ \\thenocalcul = ' + textes[1] + '\\]')
    tex_facteur_commun(valeurs, cor)


def factorisation4(cor, exo=None, valeurs=None):  #factorise 64-(x-5)²
    if not valeurs:
        valeurs = valeurs_a2moinsb2(10)
    tex_a2moinsb2(valeurs, cor, exo)


def factorisation5(cor, exo=None, valeurs=None):  # factorise a²-b²
    if not valeurs:
        valeurs = valeurs_apbamb(10)
    if valeurs[0][0] == -valeurs[1][0]:
        valeurs = ((valeurs[0][0], abs(valeurs[0][1])), (valeurs[1][0],
                   abs(valeurs[1][1])))
    else:
        valeurs = ((abs(valeurs[0][0]), valeurs[0][1]), (abs(valeurs[1][0]),
                   valeurs[1][1]))
    if exo:
        exo.append(u'\\[ \\thenocalcul = ' + tex_trinome(dev(valeurs)) + '\\]')
    cor.append(u'\\[ \\thenocalcul = ' + tex_trinome(dev(valeurs)) + '\\]')
    cor.append(u'\\[ \\thenocalcul = ' + tex_dev1(valeurs) + '\\]')
    cor.append(u'\\[ \\boxed{\\thenocalcul = ' + tex_dev0(valeurs) + '} \\]')


def tex_factorisation6(valeurs):  # renvoie les deux premieres etapes de la factorisation 6
    if valeurs[0][0] == valeurs[0][1]:  # le carre est en premier
        if valeurs[3] == 0:  # on insere un signe +
            return (tex_trinome_alea(dev(valeurs[0])) + '+' + tex_dev0(valeurs[1]),
                    tex_trinome(dev(valeurs[0])) + '+' + tex_dev0(valeurs[1]))
        else:
            return (tex_trinome_alea(dev(valeurs[0])) + '-' + tex_dev0(valeurs[1]),
                    tex_trinome(dev(valeurs[0])) + '-' + tex_dev0(valeurs[1]))
    else:
        if valeurs[2] == 0:  # on insere un signe +
            return (tex_dev0(valeurs[0]) + tex_trinome_alea(dev(valeurs[1]),
                    bplus=1), tex_dev0(valeurs[0]) + tex_trinome(dev(valeurs[1]),
                    bplus=1))
        else:
            return ('-' + tex_dev0(valeurs[0]) + tex_trinome_alea(dev(valeurs[1]),
                    bplus=1), '-' + tex_dev0(valeurs[0]) + tex_trinome(dev(valeurs[1]),
                    bplus=1))


def factorisation6(cor, exo=None, valeurs=None):  # factorise (ax+b)(cx+d)+2abx+a²x²+b²
    if not valeurs:
        valeurs = valeurs_facteurcarre(10)
    textes = tex_factorisation6(valeurs)
    if exo:
        exo.append(u'\\[ \\thenocalcul = ' + textes[0] + '\\]')
    cor.append(u'\\[ \\thenocalcul = ' + textes[0] + '\\]')
    cor.append(u'\\[ \\thenocalcul = ' + textes[1] + '\\]')
    tex_facteur_commun((valeurs[0], valeurs[1], valeurs[2], valeurs[3]),
                       cor)


#
# ------------------- DEV-FACT-EQUAT -------------------


def choix_exo(a):  # renvoie un tuple contenant les valeurs
    if a == 0:
        return valeurs_facteur_commun(10)
    elif a == 1:
        return valeurs_facteur1(10)
    elif a == 2:
        return valeurs_facteurcarre(10)
    elif a == 3:
        return valeurs_facteura2mb2(10)
    else:
        return valeurs_facteurcarre(10)


def valeur_quotient():  # renvoie un tuple contenant la valeur du quotient à calculer
    (a, b) = (random.randrange(-10, 1), random.randrange(1, 11))
    return (a // pgcd(a, b), b // pgcd(a, b))


def tex_initial(exos, a):  # renvoie l'écriture de l'expression A
    if exos == 0 or exos == 2:
        return tex_facteur_commun0(a)
    elif exos == 1:
        return tex_factorisation1(a)
    elif exos == 3:
        return tex_factorisation3(a)[0]
    else:
        return tex_factorisation6(a)[0]


def fin_fact(a):  # renvoie un tuple contenant la version factorisee de a
    return facteur_commun3(facteur_commun(a[0], a[1]), a[2], a[3])


def tex_equations(fact, cor):  # renvoie un tuple contenant les deux binomes egaux a 0
    cor.append(u'Nous savons que $A=%s$. Nous devons donc résoudre $%s=0$.\\par' %
             (tex_dev0(fact), tex_dev0(fact)))
    cor.append('Un produit de facteurs est nul signifie qu\'un des facteurs est nul. Donc :')
    cor.append('\\[%s=0 \\qquad\\text{ou}\\qquad %s=0\\]' % (tex_binome(fact[0]),
             tex_binome(fact[1])))
    eq = equations1(fact)
    if not isinstance(eq, tuple):
        cor.append("\\fbox{Cette équation n'admet aucune solution.}")
    elif not isinstance(eq[0], tuple):
        cor.append('\\[ %s=%s \\]' % (tex_coef(eq[1][0], 'x'), eq[1][1]))
        if eq[1][0] != 1:
            cor.append('\\[ x=%s \\]' % tex_frac(equations2(eq)[1]))
        cor.append(u'\\fbox{La solution de cette équation est \\,$%s$\\,.}' %
                 tex_frac(equations3(eq)[1]))
    elif not isinstance(eq[1], tuple):
        cor.append('\\[ %s=%s \\]' % (tex_coef(eq[0][0], 'x'), eq[0][1]))
        if eq[0][0] != 1:
            cor.append('\\[ x=%s \\]' % tex_frac(equations2(eq)[0]))
        cor.append(u'\\fbox{La solution de cette équation est \\,$%s$\\,.}' %
                 tex_frac(equations3(eq)[0]))
    else:
        cor.append('\\[%s=%s \\qquad\\text{ou}\\qquad %s=%s\\]' % (tex_coef(eq[0][0],
                 'x'), eq[0][1], tex_coef(eq[1][0], 'x'), eq[1][1]))
        if eq[0][0] != 1 or eq[1][0] != 1:
            cor.append('\\[x=%s \\qquad\\text{ou}\\qquad x=%s\\]' %
                     (tex_frac(equations2(eq)[0]), tex_frac(equations2(eq)[1])))
        cor.append(u'\\fbox{Les solutions de cette équation sont \\,$%s\\,\\text{ et }\\,%s$\\,.}' %
                 (tex_frac(equations3(eq)[0]), tex_frac(equations3(eq)[1])))


def equations1(a):  #renvoie ((9,5),(3,-7)) pour pouvoir ecrire 9x=5 ou 3x=-7
    if a[0][0] == 0:
        if a[1][0] == 0:
            return ''
        else:
            return ('', (a[1][0], -a[1][1]))
    elif a[1][0] == 0:
        return ((a[0][0], -a[0][1]), '')
    else:
        return ((a[0][0], -a[0][1]), (a[1][0], -a[1][1]))


def equations2(a):  # renvoie ((5,9),(-7,3)) pour pouvoir ecrire x=5/9 ou x=-7/3
    if not isinstance(a, tuple):
        return ''
    elif not isinstance(a[0], tuple):
        return ('', (a[1][1], a[1][0]))
    elif not isinstance(a[1], tuple):
        return ((a[0][1], a[0][0]), '')
    else:
        return ((a[0][1], a[0][0]), (a[1][1], a[1][0]))


def equations3(a):  # renvoie les solutions éventuellement simplifiée
    a = equations2(a)
    if not isinstance(a, tuple):
        return ''
    elif not isinstance(a[0], tuple):
        if simplifie(a[1]):
            return ('', simplifie(a[1]))
        else:
            return a
    elif not isinstance(a[1], tuple):
        if simplifie(a[0]):
            return (simplifie(a[0]), '')
        else:
            return a
    else:
        if simplifie(a[0]):
            if simplifie(a[1]):
                return (simplifie(a[0]), simplifie(a[1]))
            else:
                return (simplifie(a[0]), a[1])
        elif simplifie(a[1]):
            return (a[0], simplifie(a[1]))
        else:
            return a


def developpements(expr, exos, cor):  # effectue les developpements
    if exos != 3 and exos != 6:
        tex_developpe2(expr[0], expr[2], expr[1], expr[3], cor)
    elif exos == 6 and expr[0][0] == expr[0][1] or exos == 3 and expr[0][0][0] == \
        expr[0][1][0] and expr[0][0][1] == -expr[0][1][1]:
        cor.append(u'\\[ \\thenocalcul = ' + tex_initial(exos, expr) + '\\]')
        if expr[2]:
            siga = '-'
        else:
            siga = ''
        if expr[3]:
            sigb = '-'
        else:
            sigb = ''
        cor.append(u'\\[ \\thenocalcul = ' + '%s%s%s%s' % (siga, tex_trinome(dev(expr[0])),
                  sigb, tex_dev1(expr[1], bpar=expr[3], bplus=expr[3] -
                  1)) + '\\]')
        if expr[2]:
            a = coef_opposes(dev(expr[0]))
        else:
            a = dev(expr[0])
        if expr[3]:
            b = coef_opposes(dev(expr[1]))
        else:
            b = dev(expr[1])
        cor.append(u'\\[ \\thenocalcul = ' + tex_trinome(a) + tex_trinome(b, bplus=1) + '\\]')
        cor.append(u'\\[ \\boxed{\\thenocalcul = ' + tex_trinome(somme_polynomes(a, b)) + '} \\]')
    else:
        cor.append(u'\\[ \\thenocalcul = ' + tex_initial(exos, expr) + '\\]')
        if expr[2]:
            siga = '-'
        else:
            siga = ''
        if expr[3]:
            sigb = '-'
        else:
            sigb = ''
        cor.append(u'\\[ \\thenocalcul = ' + '%s%s%s%s' % (siga, tex_dev1(expr[0], bpar=expr[2],
                  bplus=expr[2] - 1), sigb, tex_trinome(dev(expr[1]),
                  bpar=expr[3], bplus=expr[3] - 1)) + '\\]')
        if expr[2]:
            a = coef_opposes(dev(expr[0]))
        else:
            a = dev(expr[0])
        if expr[3]:
            b = coef_opposes(dev(expr[1]))
        else:
            b = dev(expr[1])
        cor.append(u'\\[ \\thenocalcul = ' + tex_trinome(a) + tex_trinome(b, bplus=1) + '\\]')
        cor.append(u'\\[ \\boxed{\\thenocalcul = ' + tex_trinome(somme_polynomes(a, b)) + '} \\]')


def version_developpee(expr):  # renvoie la version developpe de A
    if expr[2]:
        a = coef_opposes(dev(expr[0]))
    else:
        a = dev(expr[0])
    if expr[3]:
        b = coef_opposes(dev(expr[1]))
    else:
        b = dev(expr[1])
    return somme_polynomes(a, b)


def tex_fractions(expr, nb, cor):  # repond a la question sur la valeur de x
    a = version_developpee(expr)
    cor.append('Nous savons que $A=%s$\\,. Donc pour $x=%s$\\, : ' %
             (tex_trinome(a), tex_frac(nb)))
    cor.append(u'\\[ \\thenocalcul = ' + tex_valeurx0(a, nb) + '\\]')
    if nb == (0, 1):
      cor.append(u'\\[ \\boxed{\\thenocalcul = ' + tex_valeurx1(a, nb) + '} \\]')
    else:
      cor.append(u'\\[ \\thenocalcul = ' + tex_valeurx1(a, nb) + '\\]')
    b = decomp_prod((a[0], 1), (nb[0] ** 2, nb[1] ** 2))[0:2]
    c = decomp_prod((a[1], 1), (nb[0], nb[1]))[0:2]
    a = (produit(b[0], b[1]), produit(c[0], c[1]), (a[2], 1))
    if a[0][1] == a[1][1] == 1:
      cor.append(u'\\[ \\boxed{\\thenocalcul = ' + tex_valeurx2(a, nb) + '} \\]')
    else:
      cor.append(u'\\[ \\thenocalcul = ' + tex_valeurx2(a, nb) + '\\]')
    cor.append(u'\\[ \\boxed{\\thenocalcul = ' + tex_valeurx3(a, nb) + '} \\]')


def tex_valeurx0(a, nb):
    if nb == (0, 1):
        return '%s%s%s' % (tex_coef(a[0], '\\times %s^2' % tex_frac(nb)),
                           tex_coef(a[1], '\\times %s' % tex_frac(nb),
                           bplus=a[0]), tex_coef(a[2], '', bplus=a[0] or
                           a[1]))
    else:
        return '%s%s%s' % (tex_coef(a[0], '\\times\\left(%s\\right)^2' %
                           tex_frac(nb)), tex_coef(a[1],
                           '\\times\\left(%s\\right)' % tex_frac(nb),
                           bplus=a[0]), tex_coef(a[2], '', bplus=a[0] or
                           a[1]))


def tex_valeurx1(a, nb):
    if nb == (0, 1):
        return tex_coef(a[2], '')
    elif nb[1] == 1:
        return tex_coef(a[0] * nb[0] ** 2, '') + tex_coef(a[1] * nb[0],
                '', bplus=a[0]) + tex_coef(a[2], '', bplus=a[0] or a[1])
    else:
        texte = ''
        if a[0] != 0:
            a0 = ((a[0], 1), (nb[0] ** 2, nb[1] ** 2))
            if isinstance(decomp_prod(a0[0], a0[1])[2], tuple):
                texte = tex_coef(1, tex_decomp_prod(decomp_prod(a0[0],
                                 a0[1])))
            else:
                texte = tex_coef(1, tex_frac(produit(a0[0], a0[1])))
        if a[1] != 0:
            a1 = ((a[1], 1), (nb[0], nb[1]))
            if isinstance(decomp_prod(a1[0], a1[1])[2], tuple):
                texte = texte + tex_coef(1, tex_decomp_prod(decomp_prod(a1[0],
                        a1[1])), bplus=a[0])
            else:
                texte = texte + tex_coef(1, tex_frac(produit(a1[0], a1[1])),
                        bplus=a[0])
        texte = texte + tex_coef(a[2], '', bplus=1)
        return texte


def tex_valeurx2(a, nb):
    if nb == (0, 1):
        return ''
    else:
        if a[0][1] == a[1][1] == 1:
            return tex_coef(a[0][0] + a[1][0] + a[2][0], '')
        else:
            c = den_com0(a[0], a[1])
            texte = ''
            if a[0][0] != 0:
                texte = tex_coef(1, tex_frac((a[0][0] * c[0], a[0][1] *
                                 c[0])))
            if a[1][0] != 0:
                texte = texte + tex_coef(1, tex_frac((a[1][0] * c[1], a[1][1] *
                        c[1])), bplus=texte)
            if a[2][0] != 0:
                texte = texte + tex_coef(1, tex_frac(((a[2][0] * a[0][1]) *
                        c[0], a[0][1] * c[0])), bplus=texte)
            return texte


def tex_valeurx3(a, nb):
    if nb == (0, 1) or a[0][1] == a[1][1] == 1:
        return ''
    else:
        c = den_com0(a[0], a[1])
        b = (a[0][0] * c[0] + a[1][0] * c[1] + (a[2][0] * a[0][1]) * c[0],
             a[0][1] * c[0])
        if simplifie(b):
            return tex_frac(b) + '=' + tex_frac(simplifie(b))
        else:
            return tex_frac(b)

def tex_devfacteq():
    exos = [0, 1, 2, 3, 6][random.randrange(5)]
    expr = choix_exo(exos)
    valeurx = valeur_quotient()
    fact = fin_fact(expr)
    exo = ['\\exercice']
    exo.append('On donne $A=%s$\\,.' % tex_initial(exos,
             expr))
    exo.append('\\begin{enumerate}')
    exo.append(u'\\item Développer et réduire $A$\\,.')
    exo.append('\\item Factoriser $A$\\,.')
    exo.append('\\item Calculer $A$ pour $x=%s$\\,.' % \
             tex_frac(valeurx))
    exo.append(u"\\item Résoudre l'équation $A=0$\\,.")  # % tex_dev0(fact))
    exo.append('\\end{enumerate}')
    cor = ['\\exercice*']
    cor.append('On donne $A=%s$\\,.' % tex_initial(exos,
             expr))
    cor.append('\\begin{enumerate}')
    cor.append(u'\\item Développer et réduire $A$\\,.')
    developpements(expr, exos, cor)
    cor.append('\\item Factoriser $A$\\,.')
    exec('factorisation' + str(exos) + '(cor,valeurs=expr)')
    cor.append('\\item Calculer $A$ pour $x=%s$\\,.\\par' % \
             tex_frac(valeurx))
    tex_fractions(expr, valeurx, cor)
    cor.append(u"\\item Résoudre l'équation $A=0$\\,.\\par")  # % tex_dev0(fact))
    tex_equations(fact, cor)
    cor.append('\\end{enumerate}')
    return (exo, cor)

def tex_developpements():
    nb_exos = 4
    liste_exos = (valeurs_apb2,
                  valeurs_amb2,
                  valeurs_apbamb,
                  valeurs_distr)
    exos = []
    ordre_exos = [i for i in range(nb_exos)]
    for i in range(nb_exos):
        a = random.randrange(nb_exos - i)
        exos.append(liste_exos[ordre_exos.pop(a)](10))
    exo = ['\\exercice']
    exo.append(u'Développer et réduire les expressions suivantes.')
    exo.append('\\begin{multicols}{2}\\noindent')
    cor = ['\\exercice*']
    cor.append(u'Développer et réduire les expressions suivantes.')
    cor.append('\\begin{multicols}{2}\\noindent')
    tex_developpe1(exos[0], exo, cor)
    exo.append('\\stepcounter{nocalcul}%')
    cor.append('\\stepcounter{nocalcul}%')
    tex_developpe1(exos[1], exo, cor)
    exo.append('\\stepcounter{nocalcul}%')
    cor.append('\\columnbreak\\stepcounter{nocalcul}%')
    tex_developpe1(exos[2], exo, cor)
    exo.append('\\stepcounter{nocalcul}%')
    cor.append('\\stepcounter{nocalcul}%')
    tex_developpe1(exos[3], exo, cor)
    cor.append('\\end{multicols}%')
    exos = []
    sig = [random.randrange(2) for i in range(4)]
    ordre_exos = [i for i in range(nb_exos)]
    for i in range(nb_exos):
        a = random.randrange(nb_exos - i)
        exos.append(liste_exos[ordre_exos.pop(a)](10))
    exo.append('\\stepcounter{nocalcul}%')
    cor.append('\\stepcounter{nocalcul}%')
    tex_developpe2(exos[0], sig[0], exos[1], sig[1], cor, exo)
    exo.append('\\stepcounter{nocalcul}%')
    cor.append('\\stepcounter{nocalcul}%')
    tex_developpe2(exos[2], sig[2], exos[3], sig[3], cor, exo)
    exo.append('\\end{multicols}')
    return (exo, cor)


def tex_factorisations():
    (nb_exos, ordre_exos, ordre) = (6, [], [])
    for i in range(nb_exos):
        ordre.append(str(i))
    for i in range(nb_exos):
        ordre_exos.append(ordre.pop(random.randrange(nb_exos - i)))
    del ordre
    exo = ['\\exercice']
    exo.append('Factoriser les expressions suivantes.')
    exo.append('\\begin{multicols}{2}\\noindent')
    cor = ['\\exercice*']
    cor.append('Factoriser les expressions suivantes.')
    cor.append('\\begin{multicols}{2}\\noindent')
    for i in range(nb_exos):
        exec('factorisation' + str(ordre_exos[i]) + \
            '(cor,exo)')
        exo.append('\\stepcounter{nocalcul}%')
        cor.append('\\stepcounter{nocalcul}%')
    exo.append('\\end{multicols}')
    cor.append('\\end{multicols}')
    return (exo, cor)



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
from time import strftime
from os.path import normpath, dirname, exists, abspath, join
from os import environ, name
from sys import executable, getfilesystemencoding
import sys
#import troisiemes.troisiemes, quatriemes.quatriemes, cinquiemes.cinquiemes
#import sixiemes.sixiemes, lycee.lycee

def we_are_frozen():
    """Returns whether we are frozen via py2exe.
    This will affect how we find out where we are located."""
    return hasattr(sys, "frozen")

def data_dir():
    """Renvoie le dossier data, selon qu'on utilise pyromaths à partir des
    sources, de l'exécutable win32 ou du paquet deb"""
    if we_are_frozen():
        return join(normpath(dirname(unicode(executable,
            getfilesystemencoding()))), 'data')
    elif exists(join(abspath(dirname(__file__)),'../data/')):
        return normpath(join(abspath(dirname(__file__)),'../data/'))
    else:
        return '/usr/share/pyromaths/'

def icon_dir():
    """Renvoie le dossier où se trouve l'icône, selon qu'on utilise pyromaths à
    partir des sources, de l'exécutable win32 ou du paquet deb"""
    if we_are_frozen() or exists(join(abspath(dirname(__file__)),'../data/')):
        return join(DATADIR, 'images', 'pyromaths.png')
    else:
        return join('/usr/share/pixmaps', 'pyromaths.png')
        
if name == 'nt':
    def home():
        return unicode(environ['USERPROFILE'], getfilesystemencoding())
    def configdir():
        return join(unicode(environ['APPDATA'], getfilesystemencoding()),
                "pyromaths")
elif sys.platform == "darwin":  #Cas de Mac OS X.
    def home():
        return unicode(environ['HOME'], getfilesystemencoding())
    def configdir():
        return join(home(), "Library", "Application Support", "Pyromaths")
else:
    def home():
        try:
            return unicode(environ['HOME'], getfilesystemencoding())
        except KeyError:
            # Pyromaths en ligne, l'user apache n'a pas de $HOME
            return ""
    def configdir():
        return join(home(), ".config", "pyromaths")

VERSION = '11.05.1b2'
COPYRIGHT_YEAR = strftime('%Y')
COPYRIGHTS = u'© 2006 – %s Jérôme Ortais<br/>\n' \
        u'<span style=" font-size:small;">Pyromaths est distribué sous ' \
        u'licence GPL.</span>' % (COPYRIGHT_YEAR)
WEBSITE = 'http://www.pyromaths.org/'
DATADIR = data_dir()
ICONDIR = icon_dir()
HOME = home()
CONFIGDIR = configdir()

LESFICHES = [[u'Sixième', '', [
u'Calcul mental',
u'Écrire un nombre décimal',
u'Placer une virgule',
u'Écriture fractionnaire ou décimale',
u'Décomposition de décimaux',
u'Conversions unités',
u"Conversions unités d'aires",
u"Conversions unités de volumes",
u'Poser des opérations (sauf divisions)',
u'Produits, quotients par 10, 100, 1000',
u'Classer des nombres décimaux',
u'Droites, demi-droites, segments',
u'Droites perpendiculaires et parallèles',
u'Propriétés sur les droites',
u'Multiples de 2, 3, 5, 9, 10',
u'Fractions partage',
u'Fractions et abscisses',
u'Aires et quadrillage',
u'Symétrie et quadrillages',
u'Mesurer des angles',
u'Représentation dans l\'espace',
u'Arrondir des nombres décimaux'
]],
[u'Cinquième', '', [
u'Priorités opératoires',
u'Symétrie centrale',
u'Fractions égales',
u'Sommes de fractions',
u'Produits de fractions',
u'Repérage',
u'Addition de relatifs',
u'Construction de triangles',
u'Construction de parallélogrammes',
u'Échelles',
u'Aire de disques',
u'Représentation de données',
]],
[u'Quatrième', '', [
u'Calcul mental',
u'Sommes de fractions',
u'Produits et quotients de fractions',
u'Fractions et priorités',
u'Bases du calcul littéral',
u'Réduire des expressions littérales',
u'Propriétés sur les puissances',
u'Propriétés sur les puissances de 10',
u'Écritures scientifiques',
u'Puissances de 10',
u'Distributivité',
u'Double distributivité',
u'Théorème de Pythagore',
u'Réciproque du théorème de Pythagore',
u'Cercle et théorème de Pythagore',
u'Théorème de Thalès',
u'Trigonométrie',
]],
[u'Troisième', '', [
u'Fractions',
u'Puissances',
u'PGCD',
u'Développements',
u'Factorisations',
u'Dévt, factorisat°, calcul et éq° produit',
u'Équation',
u'Racines carrées',
u'Système d\'équations',
u'Fonctions affines',
u'Probabilités',
u'Théorème de Thalès',
u'Réciproque du théorème de Thalès',
u'Trigonométrie',
u'Arithmétique'
]],
[u'Lycée', '', [
u'Équations 2° degré',
u'Factorisations 2° degré',
u'Factorisations degré 3',
u'Étude de signe',
u"Sens de variations",
u"Étude de fonctions",
u"Vecteurs",
], [
u'Niveau 1èreS',
u'Niveau 1èreS',
u'Niveau 1èreS',
u'Niveau 1èreS',
u"Niveau 1èreS, Term STG",
u"Niveau Term S",
u"Niveau Seconde",
]],
]




#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#
import re

if __name__=="__main__":
    import sys
    sys.path.append('..')
from ..outils.Affichage import decimaux

_POLYNOME_FORMAT = re.compile(r"""
    \s*                                         # éventuellement des espaces pour commencer
    (                                           # chaque monôme
        [-+]?\d+\.?\d*(?:e[-+]\d+)?[a-df-hkmnp-zA-NP-Z]?(?:\^\d+)?   # au moins un nombre décimal
        |                                       # OU
        [-+]?\d*\.?\d*(?:e[-+]\d+)?[a-df-hkmnp-zA-NP-Z](?:\^\d+)?    # au moins une variable

    )
    \s*                                         # éventuellement des espaces pour finir
""", re.VERBOSE)

_MONOME_FORMAT = re.compile(r"""
    \s*                                         # éventuellement des espaces pour commencer
    (?P<sign>[-+]?)                             # un éventuel signe
    (?P<coef>\d*\.?\d*(?:e[-+]\d+)?)                     # un éventuel coefficient
    (?:(?P<var>[a-df-hkmnp-zA-NP-Z])(?:\^(?P<deg>\d+))?)?   # une éventuelle variable avec son éventuel degré
    \s*                                         # éventuellement des espaces pour finir
""", re.VERBOSE)

class Polynome():
    """Cette classe crée la notion de polynômes.
    Polynome([[2,2],[3,1],[4,0]], 'z') est équivalent à 2z^2+3z+4.

    Un polynôme peut aussi être produit à l'aide d'une chaîne de la forme
    Polynome("2y^2+3y+4")

    Les variables ne peuvent être : e, i, j, l, o, O.
    """

    def __init__(self, monomes, var=None):
        """Crée un polynôme.
        Accepte une chaîne de caractères comme '2x^2-4x+6' ou une liste de
        monômes [coefficient, degré] finit par une string contenant la variable.
        Par exemple, Polynome([[5,2], [-4,1], [6,0]], 't') équivaut à
        Polynome("+5t^2-4t+6")
        """
        monomes = monomes or '0' # monômes du polynôme, par défaut un polynôme nul
        if isinstance(monomes, basestring):
            # Gère la construction des polynôme à partir d'une chaîne de caractères
            listmonomes = []
            for monome in _POLYNOME_FORMAT.finditer(monomes):
                m = _MONOME_FORMAT.search(monome.group(0))
                if m is None:
                    raise ValueError(u'chaîne invalide pour un objet Polynôme : %s' % input)
                m_sign = m.group('sign') or '+'
                m_coef = float(m.group('coef') or 1)
                m_deg = int(m.group('deg') or (m.group('var') and '1') or '0')
                m_var = m.group('var') or var
                if m_var and not var: var = m_var # attribue une variable à var
                if m_var != var:
                    raise ValueError(u'Le nom de la variable (%s) est incorrect pour le Polynôme %s' % (var, monomes))
                if m_sign == '-': m_coef = -m_coef
                if m_coef:
                    # supprime les monômes de coefficient 0
                    listmonomes.append([m_coef, m_deg])
            if listmonomes: self.monomes = listmonomes
            else: self.monomes = [[0, 0]]
            self.var = var or 'x'
        else:
            #Supprime les monomes nuls :
            for k in range(len(monomes)-1, -1, -1):
                if not monomes[k][0]: monomes.pop(k)
            if not monomes: monomes=[[0, 0]]
            self.monomes = monomes
            self.var = var or 'x' # Variable par défaut

    def __repr__(self):
        """repr(self)
        Renvoie une chaîne de caractère pouvant être utilisée pour créer un polynôme.
        S'appelle ainsi : repr(p) où p est un polynôme"""
        return "Polynome(%s, \"%s\")" % (self.monomes, self.var)

    def __str__(self):
        """str(self)
        Renvoie une version LaTeX du polynôme"""
        var = self.var
        s = ""
        for m in self.monomes:
            if isinstance(m, list):
                if m[1] > 1:
                    # Monôme de degré au moins 2
                    if m[0] == 1:
                        s = s + "+" + var + "^{" + str(m[1]) + "}"
                    elif m[0] == -1:
                        s = s + "-" + var + "^{" + str(m[1]) + "}"
                    elif m[0] > 0:
                        s = s + "+" +decimaux(m[0], 1) + r"\," + var + "^{" + str(m[1]) + "}"
                    else:
                        s = s + decimaux(m[0], 1) + r"\," + var + "^{" + str(m[1]) + "}"
                elif m[1] == 1:
                    # Monôme de degré 1
                    if m[0] == 1:
                        s = s + "+" + var
                    elif m[0] == -1:
                        s = s + "-" + var
                    elif m[0] > 0:
                        s = s + "+" + decimaux(m[0], 1) + r"\," + var
                    else:
                        s = s + decimaux(m[0], 1) + r"\," + var
                else:
                    # Monôme de degré 0
                    if m[0] < 0:
                        s = s + decimaux(m[0], 1)
                    else:
                        s = s + "+" + decimaux(m[0], 1)
        # supprime le + en début de séquence
        s = s.lstrip("+")
        if not s: s="0"
        return s

    def __getitem__(self, i):
        """Renvoie le i ème monôme"""
        return self.monomes[i]

    def __iadd__(self, other):
        """Définit @self += @other"""
        if isinstance(other, (float, int)):
            other = Polynome([[other, 0]], self.var)
        m1, m2 = [m for m in self.monomes], [m for m in other.monomes]
        if Polynome.degre(self) <= 0:
            self.var = other.var
        elif Polynome.degre(other)<=0:
            other.var = self.var
        if self.var != other.var:
            raise ValueError(u'Pyromaths ne sait additionner que deux polynômes de même variable')
        else :
            m=[]
            m1.extend(m2)
            for monomes in m1:
                if monomes[0]: m.append(monomes)
            return Polynome(m, self.var)

    def __eq__(self, other):
        """Renvoie True si @self est égal à @other, False sinon. Ne tient pas
        compte de l'ordre des monômes"""
        if not isinstance(other, Polynome):
            other = Polynome(other, self.var)
        return not (self.var != other.var or \
                    sorted(self.monomes, key = lambda x: (-x[1], x[0])) != \
                    sorted(other.monomes, key = lambda x: (-x[1], x[0])))

    def __ne__(self, other):
        """ @self différent de @other ?"""
        return not (self==other)

    def __add__(self, other):
        """Calcule la somme des polynômes @self et @other"""
        if isinstance(other,  basestring):
            other = eval(other)
        self += other
        return self

    def __radd__(self, other):
        if isinstance(other,  basestring):
            other = eval(other)
        if isinstance(other, (float, int)):
            other = Polynome([[other, 0]], self.var)
        return other + self

    def __sub__(self, other):
        """Jamais utilisé !"""
        if not isinstance(other, Polynome):
            self.monomes.append([-other, 0])
            return self
        elif len(other) == 1 and other.monomes[0][0]>0:
            #Cas où on soustrait un polynôme de longueur 1 et de coef positif
            return self + (-other)
        return "%r+%r" %(self, -other)

    def __rsub__(self, other):
        """Jamais utilisé !"""
        if not isinstance(other, str):
            other=str(other)
        return other + -self

    def __neg__(self):
        """Renvoie l'opposé d'un polynôme"""
        m = [m1 for m1 in self.monomes]
        for i in range(len(m)):
            m[i][0] = -m[i][0]
        return Polynome(m, self.var)

    def __pos__(self):
        """+a"""
        return self

    def __mul__(self,  other):
        #TODO: Cas d'un produit par 0 ou 1
        if not isinstance(other, Polynome):
            other=Polynome(repr(other), self.var)
        m1, m2 = [m for m in self.monomes], [m for m in other.monomes]
        if Polynome.degre(self) == 0:
            self.var = other.var
        elif Polynome.degre(other)==0:
            other.var = self.var
        if self.var != other.var:
            raise ValueError(u'Pyromaths ne sait multiplier que deux polynômes de même variable')
        p0, p1 = Polynome.reduit(self), Polynome.reduit(other)
        if len(p0) == len(p1) == 2 and (p0[0]==p1[0] or p0[1] == p1[1]) and\
            ((p0[0][0] == -p1[0][0] and p0[0][1] == p1[0][1]) or \
            (p0[1][0] == -p1[1][0]  and p0[1][1] == p1[1][1])):
            # 3ème identité remarquable
            if p0[0]==p1[0]:
                return "%r**2-%r**2" % (Polynome([p0[0]], self.var), \
                                        Polynome([[abs(p0[1][0]), p0[1][1]]],\
                                                self.var))
            else:
                return "%r**2-%r**2" % (Polynome([p0[1]], self.var), \
                                        Polynome([[abs(p0[0][0]), p0[0][1]]], \
                                                self.var))
        elif len(m1)>1 or len(m2)>1:
            p0, p1 = Polynome.reduit(self), Polynome.reduit(other)
            if p0!=self or p1 != other:
                if isinstance(p0, Polynome): p0 = repr(p0)
                else: p0 = "(" + p0 + ")"
                if isinstance(p1, Polynome): p1 = repr(p1)
                else: p1 = "(" + p1 + ")"
                return p0 + "*" + p1
            m = ""
            for f1 in m1:
                for f2 in m2:
                    m += repr(Polynome([f1], self.var)) + "*"
                    m += repr(Polynome([f2], self.var)) + "+"
            m = m[:-1] # suppression du dernier +
            return m
        elif not m1[0][0] or not m1[0][0]: return 0
        else:
            if m2[0][0] == 1 or (m2[0][1] == 0 and m1[0][1] == 0):
                # 3*x ou 3*4 -> 3x ou 12
                return Polynome([[m1[0][0]*m2[0][0], m1[0][1]+m2[0][1]]],self.var)
            else:
                # 3x*4 ou x*3x ou 3x*4x => 3*4*x ou 3*x*x ou 3*4*x*x
                m = []
                if m1[0][0] != 1:
                    m.append(str(m1[0][0]))
                if m2[0][0]<0: m.append("(" + str(m2[0][0]) + ")")
                else: m.append(str(m2[0][0]))
                if m1[0][1]>0: m.append("Polynome(\"%s^%s\")" % (self.var, m1[0][1]))
                if m2[0][1]>0: m.append("Polynome(\"%s^%s\")" % (self.var, m2[0][1]))
                return "*".join(m)

    def __rmul__(self,  other):
        """Multiplication de @other (qui n'est pas un polynôme) par @self (qui
        en est un)
        - Si @other est une suite de produits de monômes, et @self un polynôme
          de rang 1, on effectue toutes les multiplications en une fois
        - Sinon, on calcule d'abord @other"""
        if isinstance(other, (int, float)):
            other = Polynome(repr(other), self.var)
            return other*self
        if isinstance(other, str):
            from ..outils.Priorites3 import splitting
            ls = splitting(other)
            if ls.count("+") or ls.count("-"):
                # Réduire @other avant de faire le produit et vérifier s'il faut
                # des parenthèses autour de l'expression @other
                par, besoin = 0, False
                for k in range(len(ls)):
                    if ls[k] == "(": par +=1
                    elif ls[k] == ")": par -= 1
                    elif not par and ls[k] in "+-":
                        besoin = True
                        break
                if besoin: return "(" + other + ")*" + repr(self)
                else: return other + "*" + repr(self)
            elif len(self) > 1:
                return other + "*" + repr(self)
            else:
                index = 0
                for i in range(len(ls)):
                    if "Polynome(" in ls[i]:
                        if len(eval(ls[i]))>1:
                            return other + "*" + repr(self)
                        elif Polynome.degre(eval(ls[i]))>0:
                            index = index or i
                index = index or len(ls)
                if self.monomes[0][0] != 1:
                    ls.insert(index, "*")
                    if self.monomes[0][0] < 0: ls.insert(index, "(" + str(self.monomes[0][0]) + ")")
                    else: ls.insert(index, str(self.monomes[0][0]))
                if self.monomes[0][1] > 0:
                    ls.append("*")
                    ls.append("Polynome([%s], \"%s\")" %([1, self.monomes[0][1]], self.var))
                return "".join(ls)
        else: raise ValueError(u"Type non prévu. Bogue en perspective !")

    def __len__(self):
        "retourne le nombre de monômes d'un polynôme"
        m = [m1 for m1 in self.monomes]
        m =Polynome.ordonne(Polynome(m, self.var))
        return len(self.monomes)

    def degre(self):
        "retourne le degré d'un polynôme, -1 pour le polynôme nul"
        if self == Polynome(""): return -1
        else:
            m = [m1 for m1 in self.monomes]
            m =Polynome.ordonne(Polynome(m, self.var))
            return m.monomes[0][1]

    def __pow__(self, other):
        if len(self) == 2 and other ==2:
            a0 = self.monomes[0][0]
            b0 = self.monomes[1][0]
            p = a0*b0
            a = Polynome([[abs(a0), self.monomes[0][1]]],  self.var)
            b = Polynome([[abs(b0), self.monomes[1][1]]],  self.var)
            if p < 0:
                return "%r**2-2*%r*%r+%r**2"%(a, a, b, b)
            else:
                return "%r**2+2*%r*%r+%r**2"%(a, a, b, b)
        elif len(self)==1:
            return Polynome([[self[0][0]**other, self[0][1]*other]], self.var)
        else:
            result=self
            for i in range(other-1):
                result = eval(result*self)
            return result

    def reduit(self):
        """Renvoie une version réduite du polynôme self"""
        polynome = []
        for monome in Polynome.ordonne(self).monomes:
            if  polynome and monome[1] == polynome[-1][1]:
                polynome[-1][0] += monome[0]
            else:
                polynome.append([monome[0], monome[1]])
                #ATTENTION : polynome.append(monome) modifie self !!!
        for k in range(len(polynome)-1, -1, -1):
            if polynome[k][0] == 0: polynome.pop(k)
        return Polynome(polynome, self.var)

    def reduction_detaillee(self):
        """Ordonne puis effectue les factorisations qui permettent de réduire un
        polynôme"""
        po = Polynome.ordonne(self)
        if not Polynome.reductible(self) or repr(po) != repr(self):
            return po
        else:
            reductible = False
            var = self.var
            m = [m for m in po.monomes]
            s, p = "", "Polynome(\""
            tmp = ""
            for i in range(len(m)):
                if tmp == "":
                    tmp = "(%s" % str(m[i][0])
                elif m[i][1] == m[i-1][1]:
                    if m[i][0]>0: tmp += "+"
                    tmp += str(m[i][0])
                    reductible = True
                else:
                    if reductible:
                        tmp += ")"
                        s += "+%s*%r" % (tmp, Polynome('1%s^%s' % (var, m[i-1][1])))
                    else:
                        s += "+Polynome(\"%s%s^%s\")" % (str(m[i-1][0]), var,  m[i-1][1])
                    tmp = "(%s" % str(m[i][0])
                    reductible = False
            if reductible:
                tmp += ")"
                if m[-1][1]==0:
                    tmp = eval(tmp)
                    s += "+Polynome(\"%s\")" % tmp
                else:
                    s += "+%s*%r" % (tmp, Polynome('1%s^%s' % (var,  m[-1][1])))
            else:
                s += "+Polynome(\"%s%s^%s\")" % (str(m[-1][0]), var,  m[-1][1])
            s = s.lstrip("+") # suppression du + initial
            return s

    def reductible(self):
        """Renvoie 1 si le polynôme est réductible, 0 sinon."""
        if self != Polynome.reduit(self):
            return 1
        else:
            return 0

    def ordonne(self):
        """Ordonne les monômes du polynôme @self en écrivant en premier ceux
        de degré supérieur"""
        m1 = self.monomes
        m1 = sorted(m1, key = lambda x: (-x[1]))
        return Polynome(m1, self.var)



#!/usr/bin/python
# -*- coding: utf-8 -*-

from distutils.core import setup
import glob

from src.Values import VERSION

setup(name = "pyromaths",
    version = VERSION,
    description = "Exerciseur de mathématiques créant des fiches aux formats " \
            + "LaTeX et PDF.",
    license = "GPL",
    author = "Jérôme Ortais",
    author_email = "jerome.ortais@pyromaths.org",
    url = "http://www.pyromaths.org",
    package_dir={'pyromaths': 'src'},
    packages=['pyromaths', 'pyromaths.troisiemes', 'pyromaths.quatriemes',
        'pyromaths.cinquiemes', 'pyromaths.sixiemes', 'pyromaths.lycee',
        'pyromaths.outils', 'pyromaths.classes'],
    data_files=[
        ('share/pixmaps/', [ 'data/images/pyromaths.png' ] ),
        ('share/pyromaths/images', [ 'data/images/pyromaths-banniere.png',
            'data/images/whatsthis.png' ]
        ),
        ('share/applications', ['data/linux/pyromaths.desktop']),
        ('share/man/man1', ['data/linux/pyromaths.1']),
        (r'share/pyromaths/images/vignettes',
            glob.glob(r'data/images/vignettes/*.png')),
        (r'share/pyromaths/templates', glob.glob(r'data/templates/*.tex')),
        (r'share/pyromaths/packages', glob.glob(r'data/packages/*')),
        ],
    scripts = ["pyromaths"],
    platforms = ['unix'],
    long_description = """Pyromaths est un programme qui a pour but de créer
    des exercices type de mathématiques niveau collège et lycée ainsi que leur
    corrigé. C’est ce qu’on appelle parfois un exerciseur.  Contrairement à de
    nombreux autres projets, Pyromaths a pour objectif de proposer une
    correction véritablement détaillée des exercices proposés et pas seulement
    une solution.

    Il permet par exemple de proposer des devoirs maison aux élèves et de leur
    distribuer ensuite la correction. Il peut aussi servir à des familles afin
    qu’un élève puisse travailler un point du programme et se corriger
    ensuite."""
    )



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#


import math
import random
from ..classes.Racine import simplifie_racine

class Vecteur:

    def __init__(self, x=0, y=0):
      self.x=x
      self.y=y

    def __eq__(self, vec):
      '''Teste l'égalité entre deux vecteurs'''
      return (self.x==vec.x) and (self.y==vec.y)

    def __add__(self, vec):
      '''Addition'''
      return Vecteur(self.x+vec.x,self.y+vec.y)

    def __sub__(self, vec):
      '''Soustraction'''
      return Vecteur(self.x-vec.x,self.y-vec.y)

    def __mul__(self, c):
        '''Multiplication par un nombre''' ## Besoin du produit scalaire ou du produit vectoriel ?
        if isinstance(c, float) or isinstance(c, int):
            return  Vecteur(c*self.x,c*self.y)

    def __rmul__(self, c):
        '''Multiplication Reverse''' ## Besoin du produit scalaire ou du produit vectoriel ?
        if isinstance(c, float) or isinstance(c, int):
            return  self * c
    def __neg__(self):
        '''Négatif d'un vecteur'''
        return -1 * self

    def __div__(self, c):
        if isinstance(c, float) or isinstance(c, int):
            return Vecteur(self.x/float(c), self.y/float(c))

    def __abs__(self):
        '''Retourne la norme du vecteur sous la forme coeff,radicande où sqrt(n)=coeff*sqrt(radicande)'''
        return simplifie_racine(self.x**2 + self.y**2)

    def __str__(self):
        '''Affichage sous forme de coordonnées'''
        return '(' + str(self.x) + ',' + str(self.y) + ')'

    def normeTex(self):
        '''Affichage TeX de la racine simplifiée de la racine, sans les dollars.'''
        norme = abs(self)
        if norme[0] == 1:
            return "\sqrt{" + str(norme[1]) + "}"
        elif norme[1] == 1:
            return str(norme[0])
        else:
            return str(norme[0]) + "\sqrt{" + str(norme[1]) + "}"

def randvect(a, b):
    '''Retourne un vecteur aléatoire et s'occupe de placer l'abscisse au-dessus de a et l'ordonnée dans [0,b]. Il faut que b>=10.'''
    x = random.randint(-5,5)
    y = random.randint(-5,5)
    posx = a + max(1-x, 1) ## 1 colonne minimum d'écart
    if y>=0:
        posy = random.randint(0, math.fabs(y - 2)) + 1 ## Ajouter 1 pour éviter de coller au bas de la grille
    else:
        posy = random.randint(math.fabs(y), b) ## Comme y < 0, cela ne colle pas au bas de la grille
    return [Vecteur(x,y), posx, posy]
